/**
 * [ 瀑布流相关]
 * author by  goingta
 * date 2013-04-18
 */
define(function(require, exports, module) {
	//需要的命名空间
	$.ns("c360.utils","c360.views");
	require('common/storage');
	require('common/sync');//同步数据逻辑
	require('ui/share/main');//分享
	notification = require('common/Notification');
	var confirmDialog = require('ui/dialog/confirm/confirm'),
		ListView = require("ui/dialog/list/list");
	c360.events = c360.events || {};

	

	//静态常量
	var MAX_LENGTH = 30,
		TOP_RANGE = 200,
		FILTER_VERSION = 8,
		OPER_MAX_NUM = cNative.nativeAPIVersion>FILTER_VERSION?500:150,//一次性最多操作数据条数(针对相册内页和回收站)
		COLUMN = 4,
		MAX_NUM = 192,//每页最大显示照片数(单指相册内页和回收站) 256/64=4 每屏显示的张数为64张 最多显示256张即4屏
		MAR = 4;//左右边距+每列间隙

	var $page = null,
	    selectedCount = 0,//已选数据(照片)总数
		hadOpenNewView = false;//是否进入操作状态
	c360.operEnable = true;//操作按钮是否可用
	c360.haveData = false;//是否还有数据

	var lang = window.lang;//将全局变量存储到局部变量中,防止原形炼查找的时候查找不到,另外提高访问速度等

	var PhotoCollection = c360.views.PhotoCollection = Backbone.Collection.extend({
		initialize:function(){
			this.eventlistener();
		},
		eventlistener:function(){
			c360.events.on("storageAllAlbumData",this.storage,this);
		},
		storage:function(){
			c360.storage.set("albumsData",this);
		}
	});
	var PhotoModel = c360.views.PhotoModel = Backbone.Model;

	//计算指定N列下每列宽度
	var bdWidth = document.body.offsetWidth;
	// window.mar = (bdWidth*MAR/640);
	window.wid = (bdWidth-MAR*(COLUMN+1))/COLUMN;


	function isSelectState(){
		if($page == null){
			$page = $(".page");
		}
		return $page.hasClass("select");
	}

	/**
	 * [Photo 照片对象]
	 * @type {[Backbone.View]}
	 */
	var Photo = c360.views.Photo = Backbone.View.extend({
		className:"item-photo",
		events:{
			'tap .item-delete':'deletePhoto',
			'tap .photo':'photoClick'
		},
		initialize:function(setting){

			$.extend(this,setting);

			this.template = _.template(document.getElementById("photo-template").innerHTML);

			this.model.on("remove",this.deletePhoto,this);
			this.model.on("change:select",this.select,this);

			this.render();
		},
		photoClick:function(e){
			if(isSelectState()){
				//如果已选照片超过150张后,提示用户操作上限不能超过150
				if(selectedCount<OPER_MAX_NUM || this.model.get("select")){

					this.model.set("select",!this.model.get("select"));

					//照片被选中,抛出一个事件,相册内页和回收站会监听这个事件改变已选择的照片张数
					c360.events.trigger("photoIsSelected");
				}else{
					notification.bubble(cNative.nativeAPIVersion>FILTER_VERSION?lang.checkedOver500:lang.checkedOver150);
				}
			}else{
				//回收站的photo不具备openNewView 的功能
				if(this.openNewView && hadOpenNewView == false){
					/*
					* 1秒后,将已经打开openNewView置为初始值,放置连续点击两次调用两次openNewView
					* 注:此代码要写在调用openNewView之前,不然执行openNewView后,android下当前的activety被移到底层可执行不即时
					 */
					hadOpenNewView = true;
					setTimeout(function(){
						hadOpenNewView = false;
					},1000);

					//打开单张大图前,先把当前相册的所有照片放入localstorge里面,单张大图那边需要取这些数据,减少请求
					c360.storage.set("photoData",{data:c360.app.collection,url:'album.html?aid='+c360.albumData.aid});
					utils.gotoPage("photo.html?countIndex="+c360.storage.get("alreadyHadCount")+"&aid="+c360.albumData.aid+"&pid="+this.model.get("id")+"&count="+c360.albumData.count+"&name="+encodeURIComponent(c360.albumData.name),true,{bgColor:0xff000000});
					
				}
			}
		},
		select:function(){
			if(this.model.get("select")){
				this.$el.addClass("select");
				selectedCount++;
			}else{
				this.$el.removeClass("select");
				selectedCount--;
			}
		},
		deletePhoto:function(e){
			this.$el.empty().remove();
		},
		render:function(){
			this.$el.html(this.template(this.model.toJSON()));
		}
	});
	/**
	 * [Album 首页相册对象]
	 * @type {[Backbone.View]}
	 */
	var Album = c360.views.Album = Backbone.View.extend({
		className:"item-album",
		events:{
			'tap .photo':'photoClick',
			'tap .albumInfo':'editAlbumName',
			'tap .shareAlbum':'shareAlbum'
		},
		initialize:function(){
			this.coverTemplate = document.getElementById("covers-list").innerHTML;
			this.headerTemplate = document.getElementById("photo-template").innerHTML
			this.template = _.template(this.headerTemplate+this.coverTemplate);

			/*
			* 不加全部监听是因为,backbone内部实现原理是先触发单个属性变化的事件,然后再触发总的change
			* 所以绑了单个属性变化的监听再绑总的属性变化没什么意义(即最好二选一)
			* 这里选择单个属性一个一个的绑是为了提高渲染效率,带来的问题是代码量增多
			* by goingta 
			 */
			this.model.on("change:id",this.aidChange,this);
			this.model.on("change:name",this.albumNameChange,this);
			this.model.on("change:location",this.albumLocationChange,this);
			this.model.on("change:count",this.albumCountChange,this);
			this.model.on("change:covers",this.albumCoversChange,this);
			this.model.on("change:date",this.dateChange,this);
			this.model.on("remove",this.deleteAlbum,this);
			//这个注释不要删,用于解释上面的解释
			// this.model.on("change",this.render,this);
			
			this.render();
			
		},
		shareAlbum:function(e){
			c360.app.listView.show({aid:this.model.get("id"),count:this.model.get("count"),name:this.model.get("name"),photos:this.model.get("covers")});
			// c360.events.trigger("showListPanel",{aid:this.model.get("id"),count:this.model.get("count"),name:this.model.get("name")});
		},
		photoClick:function(e){

			if(hadOpenNewView) return;
			//1秒后,将已经打开openNewView置为初始值,放置连续点击两次调用两次openNewView
			hadOpenNewView = true;
			setTimeout(function(){
				hadOpenNewView = false;
			},1000);

			// /*
			// * 缓存封面的pids,同步数据的时候用
			// * 删除照片后,将照片的pid与这里缓存的pids数组进行比对,如果发现删除的照片pid存在于此pids数组中则需要调用getThemeInfo接口
			//  */
			// var covers = this.model.get("covers"),
			// 	pids = [];
			// for(var i=0, len = covers.length; i<len; i++){
			// 	pids.push(covers[i].pid);
			// }
			// c360.storage.set("coverListPids",pids);
			c360.events.trigger("storageAllAlbumData");
			c360.events.trigger("isFirstAlbumByAid",this.model.get("id"));

			//点击如果是更多,跳转到相册
			if($(e.currentTarget).hasClass("albumIn")){
				
				utils.gotoPage("album.html?aid="+this.model.get("id")+"&count="+this.model.get("count"),true,{showTitleBar:true});

			}else{
				//打开单张大图前,先把当前相册的封面照片放入localstorge里面,单张大图那边需要取这些数据,减少请求
				c360.storage.set("photoData",{data:this.model.get("covers"),url:'index.html',pre:this.model.get("pre"),next:this.model.get("next")});
				utils.gotoPage("photo.html?countIndex=0&aid="+this.model.get("id")+"&pid="+$(e.currentTarget).attr("data-pid")+"&count="+this.model.get("count")+"&name="+encodeURIComponent(this.model.get("name")),true,{bgColor:0xff000000});
					
			}
		},
		albumNameChange:function(){
			this.$(".albumName>.txt").text(this.model.get("name"));
		},
		albumLocationChange:function(){
			this.$(".location").text(this.model.get("location"));
		},
		albumCountChange:function(){
			var count = this.model.get("count");
			if(count){
				var txt = lang.photoNum.replace("{n}",count);
				if(lang.language == "en_US" && count<2){
					txt = lang.photoNum.replace('{n} photos',count+" photo");
				}
				this.$(".albumIn>.txt").text("("+txt+")");
			}else{
				// this.$el.empty().remove();
				c360.events.trigger("deleteAlbum:index",this.model.get("id"));
			}
			
		},
		albumCoversChange:function(){
			this.$(".list").replaceWith(_.template(this.coverTemplate,{covers:this.model.get("covers"),count:this.model.get("count")}));
		},
		dateChange:function(){
			this.$("span.date").text(this.model.get("date"));
		},
		editAlbumName:function(e){
			var	_this = this,
				oldName = this.model.get("name"),
				oldLocation = this.model.get("location"),
				aid = this.model.get("id");
			
			var template = '<div class="title"><span class="album_icon"></span>'+lang.theme+'</div>\
							<input id="themeInput" type="text" placeholder="'+lang.inputTheme+'" value="'+oldName+'" />\
							<div class="title"><span class="location_icon"></span>'+lang.location+'</div>\
							<input id="locationInput" type="text" placeholder="'+lang.inputLocation+'" value="'+oldLocation+'" />';

			confirmDialog.render({
				title:"",
				content:template,
				certain:lang.certain,
				cancel:lang.cancel
			},function(){
				var json = {aid:aid},
					albumName = this.$('#themeInput').val(),
					location = this.$("#locationInput").val();

				//相册名
				if(albumName == ""){
					notification.bubble(lang.inputOver30);
					c360.dialog.hideLoad();
					return;
				}
				json.name = albumName;

				//地理位置信息
				if(location != "" && location != oldLocation){
					json.location = location;
				}

				c360.server.setName(json,function(res){
					if(res.status == 200){

						/*
						* 修改临时相册后,如果改临时的照片总数超过500张,则会被分成N/500个相册
						* 后端返回的新相册为一个按时间倒序的数组,然后拿到数据后动态插入到面板中再把原有的临时相册删调
						 */
						if(res.data instanceof Array){
							notification.bubble(lang.over500);
							var album = null;
							for(var i = res.data.length-1;-1<i;i--){
								album = new Album({model:new Backbone.Model(res.data[i])});
								album.$el.attr("data-id",_this.model.get("id"));
								album.$el.insertAfter(_this.$el);
							}

							/*
							* 删掉原有的临时相册,注意清空事件绑定以及缓存到dom上的数据
							* zepto的empty和remove与jquery机制不一样,不一定会消除内存泄漏问题,有待研究
							 */
							_this.$el.empty().remove();
						}else{

							//返回的字段是aid,要修改成id
							res.data.id = res.data.aid;
							delete res.data.aid;
							/*
							* 注:此处日后有修改需求,需要注意此部分逻辑
							* 如果是临时相册,修改主题后,后端会返回正式相册的date和name,这个时候要把原来地理位置信息("暂无地理信息")清掉
							 */
							if(location == "" && res.data.date){
								_this.$(".location").text("");
							}
							_this.model.set(res.data);
						}

						confirmDialog.close();

					}else{
						notification.bubble(res.message);
						c360.dialog.hideLoad();
					}
				});
			},function(){

			},{
				focus:'themeInput'
			});
		},
		//独立照片组成的相册,修改主题后aid会改变
		aidChange:function(){
			var aid = this.model.get("id");
			this.$el.attr("data-id",aid);
		},
		render:function(){
			this.$el.html(this.template(this.model.toJSON()));
		},
		deleteAlbum:function(){
			this.$el.empty().remove();
		}
	});
	/**
	 * [AppView 整个瀑布流Page对象(包含header和footer)]
	 * @type {[Backbone.View]}
	 */
	var AppView = c360.views.AppView = Backbone.View.extend({
		pullAjaxing:false,//当前页是否在请求瀑布流数据
		startTime:null,//下一段数据的起始时间 
		maxNum:MAX_NUM,
		autoLoadMore:false,
		events:{
			'longTap .photo':"showFooter",
			'tap #more':'loadMoreClick',
			'tap #cancel':'cancelSelected'
		},
		initialize:function(options){
			/*
			* 合并属性,方法等
			 */
			//先合并事件,不然会被重置,合并完成后删除用户自定义事件,避免污染
			$.extend(this.events,options.userEvents||{});
			delete options.userEvents;
			$.extend(this,options);

			this.collection = new PhotoCollection();
			this.collection.on('add', this.addPhoto, this);
			
			//初始化dom
			this._initDom();
			this.initDom();

			//初始化上下加载时间点
			this.startTime = utils.getQuery("startTime");
			//如果url地址中有startTime则说明用户已经翻过页,则显示返回顶部的按钮
			if(this.startTime){
				this.$("#photo-list").before('<a class="loadmore" href="'+utils.getRootUrl()+"?"+utils.deleteUrlQueryByName("startTime")+'">'+lang.goTop+'</a>');
			}
			
			//自定义绑定的一些事件与函数
			this._initEventListener();
			this.initEventListener();

			this.listView = new listView(lang);

			var language = utils.getLanguage();
			if(c360.storage.get("lang")&& language != c360.storage.get("lang")){
				c360.storage.set("languageChanged",1);
			}else{
				c360.storage.set("languageChanged",0);
			}
			c360.storage.set("lang",language);
			//加载数据
			this.loadMore();
		},
		_initEventListener:function(){
			// c360.events.on("emptyPageList",this.emptyPageList,this);
			c360.events.on("photoIsSelected",this.photoIsSelected,this);
			c360.events.on("isFirstAlbumByAid",this.isFirstAlbumByAid,this);
			c360.events.on("deleteAlbum:index",this.deleteModelByIds,this);
			c360.events.on("appViewIsLoading",this.showLoading,this);
			c360.events.on("checkAll",this.checkAll,this);
			c360.events.on("invertSelection",this.invertSelection,this);
			$("body").on("swipeUp",$.proxy(this,'pullUp'));
			$(window).bind('scroll',$.proxy(this,'scroll'));
		},
		//默认要初始化的dom
		_initDom:function(){
			this.list = document.getElementById("photo-list");
			this.$operate = this.$("#operate");
			this.$loading = this.$(".loading");
			this.$footer = this.$("footer");
			this.$ad = this.$("#ad");
			this.$more = this.$("#more");
			this.$multiple = this.$("#multiple");
			this.$moreTxt = this.$("#more .txt");
			this.$cancel = this.$("#cancel");
		},
		/*全选*/
		checkAll:function(){
			this.collection.each(function(model){
				//已经为选中状态的,就不再选中了,减少view层渲染
				if(!model.get("select")){
					model.set({'select':true});
				}
			});
			this.selectedPhotosNumChange(selectedCount);
		},
		/*反选*/
		invertSelection:function(){
			this.collection.find(function(model){
				if(model.get("select")){
					model.set({'select':false});
				}else{
					model.set({'select':true});
				}
			});
			this.selectedPhotosNumChange(selectedCount);
		},
		//取消选中状态
		cancelSelected:function(){
			

			this.hideFooter();

			this.selectedPhotosNumChange();//还原标题等

			this.collection.find(function(model){
				if(model.get("select")){
					model.set("select",false);
				}
			});
			//已勾选照片计数器清零   2013-08-21  这里不清理,因为部分机型执行效率慢的在上面便利Collection还没执行完就会执行此代码,导致计数不正确
			// selectedCount = 0;
		},
		isOverLimit:function(){
			if(this.collection.length>OPER_MAX_NUM){
				return true;
			}
			return false;
		},
		//操作按钮,显示和隐藏footer
		operateFn:function(){
			if(this.collection.length == 0) return;
			
			this[isSelectState()?"cancelSelected":"showFooter"]();
		},
		showFooter:function(e){
			//每次进入选择功能之前清零 已选中照片数
			selectedCount = 0;
			if(!isSelectState()){

				this.$el.addClass("select");

				this.selectedPhotosNumChange(0);
			}
		},
		hideFooter:function(){

			this.$el.removeClass("select");
		},
		//删除已选中的照片
		deletePhotos:function(e){

			var collection = this.collection;
			collection.remove(collection.where({'select':true}));

			//取消选中状态
			this.cancelSelected();
			//重置按钮等其他状态
			this.refresh();
			//检查collection是否为空
			this.checkDataIsEmpty();
		},
		/*
		* 根据ID删除指定的model
		* 1.相册首页model是Album 2.回收站和相册内页是Photo
		* 如果是首页删除相册Album要修改Collection中相邻model中的pre和next值(跨相册逻辑)
		 */
		deleteModelByIds:function(ids){

			var collection = this.collection;
			if(ids instanceof Array){
				for(var i = 0,len = ids.length;i<len; i++){
					collection.remove(collection.get(ids[i]));
				}
			}else{//表明是相册
				var model = collection.get(ids),
					preModel = null,
					nextModel = null,
					index = collection.indexOf(model);
				try{
					if(index>0 && index<collection.length){
						nextModel = collection.at(index+1);
						preModel = collection.at(index-1);
						preModel.set("next",nextModel.get("id"));
						nextModel.set("pre",preModel.get("id"));
					}
				}catch(e){
					utils.log("model isn't album model");
				}
				
				collection.remove(model);
			}

			//取消选中状态
			this.cancelSelected();

			//重置按钮等其他状态
			this.refresh();

			//检查collection是否为空
			this.checkDataIsEmpty();
		},
		//添加单个照片
		addPhoto:function(model){
			var p = new this.item({model:model,openNewView:(typeof this.isOpenNewView == "undefined"?true:false)});
			p.$el.attr("data-id",model.get("id"));
			this.list.appendChild(p.el);
		},
		//获取选中照片数据
		getSelectedPhotos:function(){
			var pids = [],
				photos = [];
			this.collection.filter(function(model){
				if(model.get("select")){
					pids.push(model.get("id"));
					photos.push(model.toJSON());
				}
			});
			return {pids:pids,photos:photos};
		},
		photoIsSelected:function(){
			// var pids = this.getSelectedPhotos().pids;
			this.selectedPhotosNumChange(selectedCount);
		},
		//判断view是否进入勾选照片的状态
		isSelected:function(){
			return isSelectState();
		},
		//刷新幕布,更新高度等
		refresh:function(){
			this.hideLoading();
		},
		checkDataIsEmpty:function(){
			//没有数据,各种跳转或不跳转的特殊处理
			if(this.collection.length == 0){
				this.noPhoto();
			}
		},
		//瀑布流数据请求完成更改各个状态
		loadEnd:function(data,hasMore){
			hasMore = data.hasMore;//表示是否还有数据
			data = data.listData;//数据列表
			//有数据则更新各个UI控件的数据
			if(data.length){
				//首次进入
				if(this.collection.length == 0){
					this.$ad.hide();
					this.$more.show();

					//首次加载清空翻页已加载的数据条数
					if(!utils.getQuery("startTime")){
						c360.storage.set("alreadyHadCount",0);
					}
					
				}
				/*
				* 更新startTime
				 */
				var newDateTime = data[data.length-1].dateTime;
				this.startTime = newDateTime;

				this.collection.add(data);
				
			}

			/*
			* 没有数据了,防止第一屏数据没到约定量出现加载更多按钮
			* 如果之后的请求返回的hasMore为true则说明后面又有新数据(可能是新上传的数据)
			 */
			if(hasMore == false){
				c360.haveData = false;
				this.noMoreData();
			}else{
				c360.haveData = true;
				this.haveMoreData();
			}
			//检查collection是否为空
			this.checkDataIsEmpty();
			
			//重置按钮状态等
			this.refresh();
		},
		noMoreData:function(){
			this.$el.addClass("noData");
			this.$more.hide();
		},
		haveMoreData:function(){
			this.$el.removeClass("noData");
			this.$more.show();
		},
		loadMoreClick:function(evt){
			this.loadMore();
		},
		//瀑布流加载更多数据
		loadMore:function (limit,callback){
			if(this.pullAjaxing) return;
			//超过约定上限,刷新页面加载
			if (this.collection.length>=this.maxNum) {
				var count = c360.storage.get("alreadyHadCount");
				
				c360.storage.set("alreadyHadCount",this.collection.length+count);

				location.href = utils.getRootUrl()+"?startTime="+this.collection.last().get("dateTime")+"&"+utils.deleteUrlQueryByName("startTime");
				
			}else{

				this.showLoading();
				//扩展方法,各个实例自己实现的请求数据已经相应处理
				this.loadMoreCallback(limit,callback);

			}
		},
		backTop:function(){
			location.href = utils.getRootUrl()+"?"+utils.deleteUrlQueryByName("startTime");
		},
		//当滚动条滑动到底部后,如果用户在向上拉(监听swipeUp)则触发自动加载
		pullUp:function(){
			if(this.autoLoadMore){

				this.loadMore();

				this.autoLoadMore = false;
			}
		},
		//滚动条滚动事件,监听是否滚动到底部
		scroll:function(event){
			if(c360.haveData){
				/*
				* 当滚动条滚动到底部后,将标志位置为true
				* 标志位为true时,此时再往上拉则自动加载之后的数据
				*/
				var scrollVal = isCroll(0);
				if(scrollVal.flag) {
					//滚动到底部了,如果再往上拉,则自动加载数据
					this.autoLoadMore = true;
				}
			}
		},
		showLoading:function(){
			//标志正在进行ajax请求
			this.pullAjaxing = true;
			// this.$operate.hide();
			// this.$loading.addClass("active").show();
			this.$el.addClass("loadData");
			this.$more.text(lang.loading);
		},
		hideLoading:function(){
			this.pullAjaxing = false;
			// this.$operate.show();
			// this.$loading.removeClass("active").hide();
			this.$el.removeClass("loadData");
			this.$more.text(lang.more);
		},
		/*
		* 当collection.length会触发noPhoto
		* 处理办法
		* 1.有数据,则自动加载之后的数据
		* 2.没有数据了,显示广告位(背景)
		 */
		noPhoto:function(){
			if(c360.haveData){
				this.loadMore();
			}else{
				this.$ad.show();
				this.$more.hide();
			}
		},
		//是否翻过页
		hadPageOver:function(){
			return utils.getQuery("startTime")?true:false;
		},
		isFirstAlbumByAid:function(aid){
			var isFirstAlbum = (this.collection.indexOf(this.collection.get(aid)) == 0);
			c360.storage.set("isFirstAlbum",isFirstAlbum?"1":"0");
		},
		//以下是虚方法,暴露给具体实例以待扩充
		
		//加载数据的回调
		loadMoreCallback:function(dir){},
		//暴露给外部的初始化的dom方法
		initDom:function(){},
		//暴露给外部的初始化事件绑定
		initEventListener:function(){},
		//选择照片的张数
		selectedPhotosNumChange:function(num){}
	});
	
	var listView = ListView.extend({
		events:$.extend({
			'tap #shareAll':'shareAll',
			'tap #shareOther':'shareOther',
			'tap #checkAll':'checkAll',
			'tap #invertSelection':'invertSelection'
		},ListView.prototype.events),
		activePanel:function(json){
			if(json.count>OPER_MAX_NUM){
				notification.bubble(cNative.nativeAPIVersion>FILTER_VERSION?lang.selectedOver500:lang.selectedOver150);
				return;
			}
			this.show(json);
		},
		shareAll:function(){
			if(this.isOverLimit()){
				notification.bubble(cNative.nativeAPIVersion>FILTER_VERSION?lang.ShareOver500:lang.ShareOver150);
				return;
			}
			/*
			* photos取的是相册封面
			* 第一个相册的封面可能会大于8如果大于8张则只取八张封面传给分享接口
			 */
			var photos = this.model.get("photos");
			if(photos.length>8){
				photos = photos.slice(0,8);
			}

			c360.storage.set("shareData",{
				pids:[],
				photos:photos,
				name:this.model.get("name"),
				aids:[this.model.get("aid")],
				count:this.model.get("count"),
				url:location.href
			});
			c360.share.main.share();
			this.hide();
		},
		shareOther:function(){
			utils.gotoPage("shareAlbum.html?aid="+this.model.get("aid")+"&count="+this.model.get("count"),true,{showTitleBar:true});
		},
		checkAll:function(){
			c360.events.trigger("checkAll");
			this.hide();
		},
		invertSelection:function(){
			c360.events.trigger("invertSelection");
			this.hide();
		},
		isOverLimit:function(){
			if(this.model.get("count")>OPER_MAX_NUM){
				return true;
			}
			return false;
		},
		templateHTML:$("#list-template").length?$("#list-template").html():'<ul>\
						<li><span id="checkAll" class="btn-color" data-btn="cloudAlbum_select"><%=lang.checkAll%></span></li>\
						<li><span id="invertSelection" class="btn-color" data-btn="cloudAlbum_invertSelect"><%=lang.invertSelection%></span></li>\
						<li><span class="cancel btnGray-color"><%=lang.cancel%></span></li>\
					</ul>'
	});


	// 判断滚动条是否滑到底部
	function isCroll(num) {
	    var a = document.body.scrollHeight,
	    	b = document.documentElement.clientHeight,
			c = document.documentElement.scrollTop + document.body.scrollTop,
			num = num || 0;
		if((c+b+num >= a)){
			return {flag:true,top:c};
		} else {
			return {flag:false,top:c};
		}
	}

	document.documentElement.style.webkitUserSelect = "none";//禁止选中
	document.documentElement.style.webkitTouchCallout = "none"; //禁止弹出菜单

});


