define(function(require, exports, module) {
	require("common/storage");

	/*
	* 接受到需要同步的数据后,里面发出同步消息
	* 每个webview自己都绑定一个,getSyncData方法用于接收其他webview发出的需要同步的数据
	* 如果有需要异步请求数据,才能获取最新数据的就在次接受的时候处理(之前是异步处理在当前webview处理导致的结果是可能这个webview关掉了,不能接受到ajax的回调了)
	* 这样带来的问题是如果有需要异步处理的数据,会带来延迟感
	 */
	function syncAlbum(data){
		// cNative.nativeViewDispatchEvent({event:"sendData:sync", data:data},function(data){});
		getSyncData(data);
	}

	//如果pids中存在相册封面的pid,则修改相册数据
	function getSyncData(json){
		var aid = json.aid,
			pids = json.pids,
			count = json.count,
			hasNewPhoto = json.hasNewPhoto,
			covers = json.covers,
			nowIsInAlbum = json.nowIsInAlbum,//说明此同步消息是从相册内页发出的
			isFirstAlbum = c360.storage.get('isFirstAlbum');
		//传过来的pids包含封面pid,并且相册总数大于0,则说明要同步封面数据
		//如果删除的pids没有封面照片,则只同步相册照片总数,count为0时表示此相册没有照片了,会被直接删掉
		if((hasCoverPids(aid,pids) && count)||hasNewPhoto){
			cNative.nativeViewDispatchEvent({event:"editAlbum:index", data:{aid:aid,albumData:{count:count,covers:covers}}},function(data){});
		}else{
			if(pids.length){
				cNative.nativeViewDispatchEvent({event:"editAlbum:index", data:{aid:aid,albumData:{count:count}}},function(data){});
			}
		}
		//以下消息主要是用于从相册内页进单张大图,然后做删除同步
		//如果同步消息是从相册内页发出的,则不发改同步消息了,即他自己不用通知自己同步
		if(!nowIsInAlbum){
			cNative.nativeViewDispatchEvent({event:"deletePhotos:album", data:{aid:aid,albumData:{pids:pids,count:count}}},function(data){});
		}
		
	}

	//检测是否有相册封面的pid
	function hasCoverPids(aid,pids){
		
		/*
		* 缓存封面的pids,同步数据的时候用
		* 删除照片后,将照片的pid与这里缓存的pids数组进行比对,如果发现删除的照片pid存在于此pids数组中则需要调用getThemeInfo接口
		 */
		var albumsData = c360.storage.get("albumsData"),//取出index页缓存的所有相册数据
			covers = null
			coversPids = [];
		//根据aid取出对应的相册封面数据
		for(var j = 0,lenj=albumsData.length;j<lenj;j++){
			if(albumsData[j].id === aid){
				covers = albumsData[j].covers;
			}
		}
		//将取出的封面数据保存下来,进行比较
		if(covers){
			for(var i=0, len = covers.length; i<len; i++){
				coversPids.push(covers[i].pid);
			}
		}
		//求交集,如果有交集则说明此pids中有封面的pid
		return _.intersection(coversPids,pids).length>0?true:false;
	}	
	
	c360.events.on("syncAlbum",syncAlbum);	

	c360.events.on("sendData:sync",getSyncData);

});