define(function(require, exports, module) {

	$.fn.player = function( options ) {
		var name = "player",
			isMethodCall = typeof options === "string",
			args = Array.prototype.slice.call( arguments, 1 );

		if ( isMethodCall ) {
			this.each(function() {
				var instance = this[name];
				if(instance){
					if($.isFunction( instance[options] )){
						instance[ options ].apply( instance, args);
					}else{
						console.log('参数传入错误');
						return false;
					}
				}else{
					this[name] = new $.audio( this );
				}
			});
		}else{
			this.each(function() {
				var instance = this[name];
				if(!instance){
					this[name] = new $.audio( this );
				}
			});
		}

		return this;
	};

	$.audio = function( element ) {
		this.element = $(element);
		this.initialize();
	};

	$.audio.event = {
		ready: "ready",
		error: "error",
		play: "play",
		ended: "ended",
		loadeddata:"loadeddata"
	};

	$.audio.media = null;

	$.audio.type = 'load';

	$.audio.timer = null;

	$.audio.prototype = {

		/**********************************************
		 * @actionName  初始化audio.
		 *********************************************/
		initialize:function(){
			this._createElement();
			this.addEventListener();
		},

		/**********************************************
		 * @actionName  创建一个audio对象
		 *********************************************/
		_createElement:function(){
			this.audioElement = document.createElement('audio');
			this.element.append(this.audioElement);
		},

		/**********************************************
		 * @actionName  停止播放
		 *********************************************/
		stop: function() {
			if(this.flag == 'error') return;
			try {
				// 停止音频
				if($.audio.media) $.audio.media.stop();
			} catch(e) {
				this._trigger($.audio.event.error);
			}
		},

		/**********************************************
		 * @actionName  开始播放
		 *********************************************/
		play: function() {
			if(this.flag == 'error') return;
			try {
				// 播放音频
				if($.audio.media) $.audio.media.play();
			} catch(e) { 
				this._trigger($.audio.event.error);
			}
		},

		/**********************************************
		 * @actionName  释放音频占用的内存
		 *********************************************/
		release:function(){
			if(this.flag == 'error') return;
			this.flag = 'error';
			$.audio.timer && clearTimeout($.audio.timer);
			// 如果media对象存在,就要释放media内存,在删除media对象
			if($.audio.media) {
				$.audio.media.release();
				delete $.audio.media;
			}
		},

		/**********************************************
		 * @actionName  加载音频的失败的回调函数
		 *********************************************/
		loadAudio:function(){
			var _this = this,

				callback = function(){
					if(this.flag == 'error'){
						notification.bubble(lang.audioPlayType);
						clearTimeout($.audio.timer);
						$.audio.timer = null;
					}
				};

			$.audio.timer && clearTimeout($.audio.timer);
			$.audio.timer = setTimeout(callback,10000);
		},

		/**********************************************
		 * @actionName  加载native的Media
		 *********************************************/
		loadMedia:function(){
			// 如果已经释放了就不播放了
			if($.audio.type == 'release') return;
			if($.audio.media) $.audio.media.release();
			// 从目标文件创建Media对象
			$.audio.media = new Media(this.mediaSrc, $.proxy(this,'onMediaSuccess'), $.proxy(this,'onMediaError'), $.proxy(this,'onStatus'));    
			// 播放音频
	       	$.audio.media.play();
		},

		/**********************************************
		 * @actionName  设置html5音频loading
		 *********************************************/
		setMedia:function(src){
			this.flag = 'error';
			$.audio.type = 'load';
			this.mediaSrc = src;
			this.audioElement.autoplay = false;
			this.audioElement.src = src;
			this.audioElement.load();
			this.loadAudio();
		},

		/**********************************************
		 * @actionName  创建media对象成功回调函数,播放结束后的回调
		 *********************************************/
		onMediaSuccess:function(message){
		},

		/**********************************************
		 * @actionName  创建media对象失败回调函数,播放停止后的回调
		 *********************************************/
		onMediaError:function(message){
		},

		/**********************************************
		 * @actionName  创建media对象失败回调函数,播放开始的回调
		 *********************************************/
		onStatus:function(code){
			switch(code){
				case 2:
					this._trigger($.audio.event.play);
					break;
				case 0:
				case 4:
					this._trigger($.audio.event.ended);
					break;
				default:
					break;
			}
		},

		/**********************************************
		 * @actionName  注册trigger事件
	     * @param eventType string 表示事件名称
		 *********************************************/
		_trigger: function(eventType) {
			this.element.trigger($.Event(eventType));
		},

		/**********************************************
		 * @actionName  注册audio相关事件
		 *********************************************/
		addEventListener:function(){
			var _this = this;

			// 注册加载文件完成事件
			this.audioElement.addEventListener($.audio.event.loadeddata, function(event) {
				$.audio.timer && clearTimeout($.audio.timer);
				_this.flag = 'success';
				_this.loadMedia();
			},false);

			// 用时间延迟是为了让类实例化完成后,执行ready事件,如果没延迟会造成死循环
			this.readyTime && clearTimeout(this.readyTime);
			this.readyTime = setTimeout(function(){
				_this._trigger($.audio.event.ready);
			},0);
		}
	};
});