define(function(require, exports, module) {

    require('component/crop/crop.css');

    // 定义常量类
    var CONSTANT = {
            TOUCHFLAG       : ('ontouchstart' in this),
            BASECLASS       : 'crop',
            CREATEHANDLES   : ['nw','ne','se','sw']
        },

        crop = function (obj, opt) {
            var options = $.extend({}, crop.defaults);
            function px(n) {
                return Math.round(n) + 'px';
            }
            function cssClass(cl) {
                return CONSTANT.BASECLASS + '-' + cl;
            }
            function getPos(obj) {
                var pos = $(obj).offset();
                return [pos.left, pos.top];
            }
            function mouseAbs(e) {
                return [(e.pageX - docOffset[0]), (e.pageY - docOffset[1])];
            }
            function setOptions(opt) {
                if (typeof(opt) !== 'object') opt = {};
                options = $.extend(options, opt);

                $.each(['onChange','onSelect','onRelease'],function(i,e) {
                    if (typeof(options[e]) !== 'function') options[e] = function () {};
                });
            }
            function startDragMode(mode, pos) {
                docOffset = getPos($img);
                if (mode === 'move') {
                    return Tracker.activateHandlers(createMover(pos), doneSelect);
                }

                var fc = Coords.getFixed(),
                    opp = oppLockCorner(mode),
                    opc = Coords.getCorner(oppLockCorner(opp));
                Coords.setPressed(Coords.getCorner(opp));
                Coords.setCurrent(opc);

                Tracker.activateHandlers(dragmodeHandler(), doneSelect);
            }
            function dragmodeHandler() {
                return function (pos) {
                    Coords.setCurrent(pos);
                    Selection.update();
                };
            }
            function createMover(pos) {
                var lloc = pos;
                return function (pos) {
                    Coords.moveOffset([pos[0] - lloc[0], pos[1] - lloc[1]]);
                    lloc = pos;

                    Selection.update();
                };
            }
            function oppLockCorner(ord) {
                switch (ord) {
                    case 'ne':
                        return 'sw';
                    case 'nw':
                        return 'se';
                    case 'se':
                        return 'nw';
                    case 'sw':
                        return 'ne';
                }
            }
            function createDragger(ord){
                return function (e,event) {

                    event.preventDefault();
                    event.stopPropagation();

                    docOffset = getPos($img);

                    startDragMode(ord, mouseAbs(e));

                    return false;
                };
            }
            function unscale(c) {
                return {
                    x: c.x * xscale,
                    y: c.y * yscale,
                    x2: c.x2 * xscale,
                    y2: c.y2 * yscale,
                    w: c.w * xscale,
                    h: c.h * yscale
                };
            }
            function doneSelect() {
                var c = Coords.getFixed();

                if ((c.w >= options.minSize[0]) && (c.h >= options.minSize[1])) {
                    Selection.done();
                }
            }
            function startDrag(){
                selectFlag = false;
                docOffset = getPos($img);
                Selection.enableHandles();
                Coords.setPressed([0,0]);
                Coords.setCurrent([photoSize,photoSize]);
                Selection.update(true);
            }
            function selectDrag(pos) {
                Coords.setCurrent(pos);
                Selection.update();
            }

            setOptions(opt);

            var $img = $oldimg = $(obj),

                selectFlag = true,

                boundx = $img.width(),

                boundy = $img.height(),

                photoSize = Math.min(boundx,boundy),

                $div = $('<div />').addClass(cssClass('holder')).insertAfter($oldimg).append($img);

            if(photoSize <= 100){
                CONSTANT.CREATEHANDLES = [];
            }else{
                CONSTANT.CREATEHANDLES = ['nw','ne','se','sw'];
            }

            options.addClass && $div.addClass(options.addClass);

            var $img2 = $img.clone(),

                $img_holder = $('<div />').addClass(cssClass('img-holder')),

                $hdl_holder = $('<div />').addClass(cssClass('hdl-holder')),

                $sel = $('<div />').addClass(cssClass('panel-holder')).insertBefore($img).append($img_holder, $hdl_holder),

                bgopacity = options.bgOpacity,

                xlimit, ylimit, xmin, ymin, xscale = 1, yscale = 1, btndown = false,

                $track = $('<div />').addClass(cssClass('tracker')).bind('downtap', createDragger('move'));

            $img_holder.append($track).append($img2);

            var Coords = (function () {
                var x1 = 0,
                    y1 = 0,
                    x2 = 0,
                    y2 = 0,
                    ox, oy;

                function setPressed(pos) {
                    pos = rebound(pos);
                    x2 = x1 = pos[0];
                    y2 = y1 = pos[1];
                }

                function setCurrent(pos) {
                    pos = rebound(pos);
                    ox = pos[0] - x2;
                    oy = pos[1] - y2;
                    x2 = pos[0];
                    y2 = pos[1];
                }

                function moveOffset(offset) {
                    ox = offset[0];
                    oy = offset[1];

                    if (0 > x1 + ox) {
                        ox -= ox + x1;
                    }
                    if (0 > y1 + oy) {
                        oy -= oy + y1;
                    }

                    if (boundy < y2 + oy) {
                        oy += boundy - (y2 + oy);
                    }
                    if (boundx < x2 + ox) {
                        ox += boundx - (x2 + ox);
                    }

                    x1 += ox;
                    x2 += ox;
                    y1 += oy;
                    y2 += oy;
                }

                function getCorner(ord) {
                    var c = getFixed();
                    switch (ord) {
                        case 'ne':
                            return [c.x2, c.y];
                        case 'nw':
                            return [c.x, c.y];
                        case 'se':
                            return [c.x2, c.y2];
                        case 'sw':
                            return [c.x, c.y2];
                    }
                }

                function getFixed() {
                    if (!options.aspectRatio) {
                        return getRect();
                    }

                    var aspect = options.aspectRatio,
                        min_x = options.minSize[0] / xscale,
                        max_x = options.maxSize[0] / xscale,
                        rw = x2 - x1,
                        rh = y2 - y1,
                        rwa = Math.abs(rw),
                        rha = Math.abs(rh),
                        real_ratio = rwa / rha,
                        xx, yy, w, h;

                        if (max_x === 0) {
                            max_x = photoSize;
                        }

                        if (real_ratio < aspect) {
                            yy = y2;
                            w = rha * aspect;
                            xx = rw < 0 ? x1 - w : w + x1;

                            if (xx < 0) {
                                xx = 0;
                                h = Math.abs((xx - x1) / aspect);
                                yy = rh < 0 ? y1 - h : h + y1;
                            } else if (xx > boundx) {
                                xx = boundx;
                                h = Math.abs((xx - x1) / aspect);
                                yy = rh < 0 ? y1 - h : h + y1;
                            }
                        } else {
                            xx = x2;
                            h = rwa / aspect;
                            yy = rh < 0 ? y1 - h : y1 + h;
                            if (yy < 0) {
                                yy = 0;
                                w = Math.abs((yy - y1) * aspect);
                                xx = rw < 0 ? x1 - w : w + x1;
                            } else if (yy > boundy) {
                                yy = boundy;
                                w = Math.abs(yy - y1) * aspect;
                                xx = rw < 0 ? x1 - w : w + x1;
                            }
                        }  

                        if (xx > x1) { 
                            if (xx - x1 < min_x) {
                                xx = x1 + min_x;
                            } else if (xx - x1 > max_x) {
                                xx = x1 + max_x;
                            }
                            if (yy > y1) {
                                yy = y1 + (xx - x1) / aspect;
                            } else {
                                yy = y1 - (xx - x1) / aspect;
                            }
                        } else if (xx < x1) {
                            if (x1 - xx < min_x) {
                                xx = x1 - min_x;
                            } else if (x1 - xx > max_x) {
                                xx = x1 - max_x;
                            }
                            if (yy > y1) {
                                yy = y1 + (x1 - xx) / aspect;
                            } else {
                                yy = y1 - (x1 - xx) / aspect;
                            }
                        }

                        if (xx < 0) {
                            x1 -= xx;
                            xx = 0;
                        } else if (xx > boundx) {
                            x1 -= xx - boundx;
                            xx = boundx;
                        }

                        if (yy < 0) {
                            y1 -= yy;
                            yy = 0;
                        } else if (yy > boundy) {
                            y1 -= yy - boundy;
                            yy = boundy;
                        }

                        return makeObj(flipCoords(x1, y1, xx, yy));
                }
                function rebound(p) {
                    if (p[0] < 0) p[0] = 0;
                    if (p[1] < 0) p[1] = 0;

                    if (p[0] > boundx) p[0] = boundx;
                    if (p[1] > boundy) p[1] = boundy;

                    return [Math.round(p[0]), Math.round(p[1])];
                }
                function flipCoords(x1, y1, x2, y2) {
                    var xa = x1,
                        xb = x2,
                        ya = y1,
                        yb = y2;
                    if (x2 < x1) {
                        xa = x2;
                        xb = x1;
                    }
                    if (y2 < y1) {
                        ya = y2;
                        yb = y1;
                    }
                    return [xa, ya, xb, yb];
                }
                function getRect() {
                    var xsize = x2 - x1,
                        ysize = y2 - y1,
                        delta;

                    if (xlimit && (Math.abs(xsize) > xlimit)) {
                        x2 = (xsize > 0) ? (x1 + xlimit) : (x1 - xlimit);
                    }

                    if (ylimit && (Math.abs(ysize) > ylimit)) {
                        y2 = (ysize > 0) ? (y1 + ylimit) : (y1 - ylimit);
                    }

                    if (ymin / yscale && (Math.abs(ysize) < ymin / yscale)) {
                        y2 = (ysize > 0) ? (y1 + ymin / yscale) : (y1 - ymin / yscale);
                    }
                    if (xmin / xscale && (Math.abs(xsize) < xmin / xscale)) {
                        x2 = (xsize > 0) ? (x1 + xmin / xscale) : (x1 - xmin / xscale);
                    }

                    if (x1 < 0) {
                        x2 -= x1;
                        x1 -= x1;
                    }
                    if (y1 < 0) {
                        y2 -= y1;
                        y1 -= y1;
                    }
                    if (x2 < 0) {
                        x1 -= x2;
                        x2 -= x2;
                    }
                    if (y2 < 0) {
                        y1 -= y2;
                        y2 -= y2;
                    }
                    if (x2 > boundx) {
                        delta = x2 - boundx;
                        x1 -= delta;
                        x2 -= delta;
                    }
                    if (y2 > boundy) {
                        delta = y2 - boundy;
                        y1 -= delta;
                        y2 -= delta;
                    }
                    if (x1 > boundx) {
                        delta = x1 - boundy;
                        y2 -= delta;
                        y1 -= delta;
                    }
                    if (y1 > boundy) {
                        delta = y1 - boundy;
                        y2 -= delta;
                        y1 -= delta;
                    }

                    return makeObj(flipCoords(x1, y1, x2, y2));
                }

                function makeObj(a){
                    return {
                        x: a[0],
                        y: a[1],
                        x2: a[2],
                        y2: a[3],
                        w: a[2] - a[0],
                        h: a[3] - a[1]
                    };
                }

                return {
                    flipCoords: flipCoords,
                    setPressed: setPressed,
                    setCurrent: setCurrent,
                    moveOffset: moveOffset,
                    getCorner: getCorner,
                    getFixed: getFixed
                };
            }());

            var Selection = (function () {
                var awake,
                    hdep = 800;

                function dragDiv(ord, zi) {
                    var jq = $('<div />').css({
                            zIndex: zi
                        }).addClass('ord-'+ord).bind('downtap', createDragger(ord));

                    $hdl_holder.append(jq);
                    return jq;
                }
                function insertHandle(ord) {

                    var hs = options.handleSize,

                        div = dragDiv(ord, hdep++).addClass(cssClass('handle'));

                    if (hs) {
                        div.css({
                            width:hs,
                            height:hs
                        });
                    }

                    return div;
                }
                function createHandles(li) {
                    var i = 0;
                    for (; i < li.length; i++) {
                        insertHandle(li[i]);
                    }
                }
                function moveto(x, y) {
                    $img2.animate({
                        translateX:px(-x),
                        translateY:px(-y)
                    },0);

                    $sel.animate({
                        translateX:px(x),
                        translateY:px(y)
                    },0);
                }
                function resize(w, h) {
                    w = Math.round(Math.min(Math.max(w,0),boundx));
                    h = Math.round(Math.min(Math.max(h,0),boundy));
                    $sel.width(w).height(h);
                }
                function refresh() {

                    var c = Coords.getFixed();
                    Coords.setPressed([c.x, c.y]);
                    Coords.setCurrent([c.x2, c.y2]);

                    updateVisible();
                }
                function updateVisible(select) {
                    if (awake) {
                        return update(select);
                    }
                }
                function update(select) {
                    var c = Coords.getFixed();

                    resize(c.w, c.h);
                    moveto(c.x, c.y);

                    awake || show();

                    if (select) {
                        options.onSelect.call($oldimg, unscale(c));
                    } else {
                        options.onChange.call($oldimg, unscale(c));
                    }
                }
                function show() {
                    $sel.show();
                    $img.css({
                       opacity:bgopacity
                    });

                    awake = true;
                }
                function release() {
                    selectFlag = true;

                    disableHandles();

                    $sel.hide();

                    $img.css({
                       opacity:1 
                    });

                    awake = false;

                    options.onRelease.call($oldimg);
                }
                function enableHandles() {
                    $hdl_holder.show();
                }
                function disableHandles() {
                    $hdl_holder.hide();
                }
                function done() {
                    enableHandles();
                    refresh();
                } 

                createHandles(CONSTANT.CREATEHANDLES);

                disableHandles();

                return {
                    updateVisible: updateVisible,
                    update: update,
                    release: release,
                    refresh: refresh,
                    isAwake: function () {
                        return awake;
                    },
                    enableHandles: enableHandles,
                    disableHandles: disableHandles,
                    done: done
                };
            }());

            var Tracker = (function () {
                var onMove = onDone = function () {};

                function toFront() {
                    $div.bind('movetap.crop',move).bind('uptap.crop',up).bind('langtap.crop',up);
                }

                function toBack() {
                    $div.unbind('.crop');
                }
                function move(e,event) {
                    event.preventDefault();
                    event.stopPropagation();
                    onMove(mouseAbs(e));
                    return false;
                }
                function up(e,event) {
                    event.preventDefault();
                    event.stopPropagation();

                    if (btndown) {

                        btndown = false;

                        onDone();

                        if (Selection.isAwake()) {
                            options.onSelect.call($oldimg, unscale(Coords.getFixed()));
                        }

                        onMove = onDone = function () {};
                    }

                    toBack();

                    return false;
                }
                function activateHandlers(move, done) {
                    btndown = true;
                    onMove = move;
                    onDone = done;
                    toFront();
                    return false;
                }

                $.startDraw = function(){
                    $('.crop-tracker').bind('downtap',createDragger('move'));
                    var item = null;
                    for (var i = 0; i < CONSTANT.CREATEHANDLES.length; i++) {
                        item = CONSTANT.CREATEHANDLES[i];
                         $('.crop-'+item).bind('downtap',createDragger(item));
                    }
                }

                $.stopDraw = function(){
                    $('.crop-handle').unbind('downtap');
                    $('.crop-tracker').unbind('downtap');
                }

                return {
                    activateHandlers: activateHandlers
                };
            }());

            startDrag();

            return $oldimg;
        };

    crop.defaults = {
        addClass: null,
        bgOpacity: 0.6,
        handleSize: 60,
        aspectRatio: 0,
        maxSize: [0, 0],
        minSize: [0, 0],
        onChange: function () {},
        onSelect: function () {},
        onRelease: function () {}
    };

    $.fn.crop = function (options){
        this.each(function () {
            crop(this, options);
        });
        return this;
    };

});