/**************************************************************************/
/*                                                                        */
/* Copyright (c)2010-2012  Pinguo Company             　　　　　　　      */
/*                 品果科技                            版权所有 2010-2012 */
/*                                                                        */
/* PROPRIETARY RIGHTS of Pinguo Company are involved in the  　　　　　　 */
/* subject matter of this material.  All manufacturing, reproduction, use,*/
/* and sales rights pertaining to this subject matter are governed by the */
/* license agreement.  The recipient of this software implicitly accepts  */
/* the terms of the license.                                              */
/* 本软件文档资料是品果公司的资产,任何人士阅读和使用本资料必须获得        */
/* 相应的书面授权,承担保密责任和接受相应的法律约束.                       */
/*                                                                        */
/**************************************************************************/
define(function(require, exports, module) {

    require('component/dialog/dialog.css');

	/**********  @actionName  封装全局的dialogPanel实例  **********/
	var dialogPanel = function(config) {
		
		config = $.extend($.extend({},dialogPanel.defaults),config);

        var _this = dialogPanel.fn.singleton;
        
        // 判断是否是单例模式并是否第一次初始化弹出框
		if(_this){
			_this.constructor(config);
		}else{
			if(_this)config.zIndex = _this.config.zIndex;
			_this = dialogPanel.fn.singleton = new dialogPanel.fn.constructor(config);
		}
		return _this;
    };
    
    // 缓存弹出框内容Dom
    dialogPanel.contentCollection = new Backbone.Collection();
    
    // 构建弹出框面板DOM
	var panelDom = '<div class="d-outer">\
		<table class="d-border">\
		    <tbody>\
			    <tr>\
				    <td class="d-c">\
					    <div class="d-inner">\
						    <table>\
							    <tbody>\
								    <tr>\
									    <td class="d-main">\
									    	<div class="d-content"></div>\
									    </td>\
								    </tr>\
							    </tbody>\
						    </table>\
					    </div>\
				    </td>\
			    </tr>\
		    </tbody>\
		</table>\
	</div>';
	
	// 弹出框面板公共接口定义
    dialogPanel.fn = dialogPanel.prototype = {
    	singleton:null,
    	index:null,
    	/**********  @actionName  创造者  **********/
        constructor: function(config) {

            this.closed = false;
            
            this.config = config;
            
            this.config.content = $(this.config.content);
            
            this.config.id = this.config.content.attr('id');

            config.lock && this.lock();

            this.dom = this.dom || this.getDom();

            // 如果是单例模式就先关闭当前弹出框在打开新的弹出框
        	this.showArray = this.dom.content.children();
            this.showLen = this.isVisible(this.showArray).length;
            if(this.showLen > 0){
            	this.close($.dialog);
            	return this;
            }
            
            config.type && this.dom.outer.addClass(config.type);
            
            config.skin && this.dom.wrap.addClass(config.skin);
            
            this.dom.wrap.css('position', config.fixed ? 'fixed': 'absolute');
            
            this.content().resize().time();

            this.zIndex()

            $.isFunction(config.initialize) && config.initialize.call(this);
            
            this.addEvent();

            this[config.visible ? 'visible': 'hidden']().focus();

            return this;
        },
        
        isVisible:function(dom){
            var element = null,array = [];
            dom.each(function(){
                element = $(this);
                if(element.css('display')!='none' && element.css('visibility')!='hidden' && element.height()>0){
                    array.push(element);
                }
            });
            return array;
        },

        /********************* @actionName  设置内容   ********************/
        content: function() {
            var panel = this.dom.content,
                config = this.config,
                content = config.content,
                id = config.id;
            if(!dialogPanel.contentCollection.get(id)){
            	// 克隆新对象参数是为了实现一个实例多个弹出框切换
            	dialogPanel.contentCollection.add($.extend({},config));
                panel.append(content);
            }

            if($.isFunction(this.config.domShowCallback)){
                this.config.domShowCallback.call(this);
            }
            
            content.show();
            this.dom.wrap.show();
            return this;
        },
        
        /**********  @actionName  面板大小布局  **********************/
        resize: function() {
        	
			this.dom.main.width(this.config.width).height(this.config.height);
			
            return this;
        },

		/**********  @actionName  位置居中  **********/
        reset: function() {

            var dom = this.dom,
            $wrap = dom.wrap,
            $window = dom.window,
            ww = $window.width(),
            wh = $window.height(),
            ow = $wrap.width(),
            oh = $wrap.height();

			$wrap.css({
				left:(ww - ow) / 2,
				top:(wh - oh) / 4
			});
			
            return this;
        },

		/**********  @actionName  显示对话框  **********/
        visible: function() {

            this.dom.outer.show();

            this.reset();

            $(document.body).trigger("dialogIn");
            
            return this;
        },

        showLoad : function(){
            var warp = this.dom.wrap;

            $(".loading",warp).show();
        },

        hideLoad : function(){
            var warp = this.dom.wrap;

            $(".loading",warp).hide();

            $('.target-btn',warp).removeClass('target-btn');
        },

        /**********  @actionName  隐藏对话框  **********/
        hidden: function() {

            this.dom.outer.hide();

            return this;
        },

        /**********  @actionName  关闭对话框  **********/
        close: function(callback) {
            if(typeof(callback) == 'object'){
                if(!$(callback.target).hasClass('d-mask')) return;
            }

        	if (this.closed) return this;
        	
        	this.closed = true;
        	
        	var _this = this,config = this.config,beforeunload,afterunload;
        	
        	if(this.showLen > 0){
        		// 根据ID查找对应的面板内容信息
        		config = this.getIdByContent(this.showArray[0].id).toJSON(); 
        	}
        	
        	config.zIndex --;
        	
        	afterunload = config.afterunload;

            beforeunload = config.beforeunload;
        	
            $.isFunction(beforeunload) && beforeunload();

        	clearTimeout(this._timer);

            this.hidden();

            clearTimeout(this.closeTimer);

            config.content.hide();
            
            this.closeTimer = setTimeout(function() {

                _this.blur();
				
                _this.hideLoad();

                _this.unlock();
                
                config.type && _this.dom.outer.removeClass(config.type);
                
                _this.dom.wrap.hide();
                
            	for(var i = 0; i < _this.showLen;i++){
            		$(_this.showArray[i]).hide();
            	}
                
                _this.removeEvent();
                
                $(document.body).trigger("dialogOut");
				
                if($.isFunction(callback)){
                	callback(_this.config);
                }
                // 实现一个预留接口暂时弹出框切换，前一个弹出框不做关闭后回调函数
                if(callback !== true && $.isFunction(afterunload)){
                	afterunload();
                }
                
                return _this;
            }, config.delay);
        },
        
   		/********************* @actionName  定时关闭   ***********************/
        time: function() {
			var time = this.config.time;
            if (time) {
                this._timer = setTimeout($.proxy(this,'close'),time);
            }

            return this;
        },

        blur:function(){
            // 清空所有的input，textarea时，需要抛出blur事件
            $("input,textarea",this.dom.wrap).blur();
            
            clearTimeout(this.blurTimer);

            this.blurTimer = setTimeout(function(){
                window.scrollTo(0,document.body.scrollTop);
            }, 0);
        },

        /**********  @actionName  设置焦点  **********/
        focus: function() {
        	var focus = this.config.focus;
            
            if(!focus) return this;
            
            clearTimeout(this.focusTimer);

            this.focusTimer = setTimeout(function(){
                if($.type(focus) === 'string'){
                    $("#"+focus).focus();
                }else if($.type(focus) === 'object'){
                    focus.focus();
                }
            }, this.config.delay);

            return this;
        },

        /**********  @actionName  置顶对话框  **********/
        zIndex: function() {
            var dom = this.dom,
                zIndex = this.config.zIndex++;
            // 设置叠加高度
            dom.wrap.css('zIndex', zIndex);
            
            this._lockMask && this._lockMask.css('zIndex', zIndex);

            dom.outer.addClass('d-state-focus');

            return this;
        },

        /**********  @actionName  设置屏锁  **********/
        lock: function() {

            if (this._isLock) return this;

			if(document.getElementById('dialogMask')){
                this._lockMask = $('#dialogMask');
			}else{
				this._lockMask = $("<div id='dialogMask'></div>");
				this._lockMask.addClass('d-mask');
	            $(document.body).append(this._lockMask);
			}
            
            if(this.config.bgOpacity != null){
                this._lockMask.css({opacity:this.config.bgOpacity});
            }else{
                this._lockMask.show();
            }

            if(!this.config.rejectBackgroundClose){
                this._lockMask.bind('tap.dropevent',$.proxy(this,'close'));
            }            
			
            this._isLock = true;

            return this;
        },

        /**********  @actionName  解开屏锁  **********/
        unlock: function() {

            if (!this._isLock) return this;
			
			this._lockMask.unbind('.dropevent');

            this._lockMask.hide();

            this._isLock = false;

            return this;
        },

        /**********  @actionName  获取dom元素  **********/
        getDom: function() {
        	var dom = {},
                $body = $(document.body),
                $wrap = $(document.createElement('div')),
                name;

            // 判断加锁背景是否已经存在
            if(document.getElementById('dialogMask')){
                this._lockMask = $('#dialogMask');
                this._lockMask.append($wrap);
                
                /*
                * hack代码:
                * 解决弹出框弹出后的遮罩层会触发浏览器滚动条事件,以后等弹出框代码优化后再来处理此代码 
                * by goingta date:2013-05-16
                 */
                this._lockMask.bind("touchmove",function(e){utils.stopBubble(e)});

            }else{
                $body.append($wrap);
            }
    		
    		$wrap.html(panelDom);

    		var els = $wrap[0].getElementsByTagName('*'), elsLen = els.length;
    		for (var i = 0; i < elsLen; i++) {
                name = els[i].className.split('d-')[1];
                if (name) dom[name] = $(els[i]);
            };
        	
            dom.window = $(window);
            
            dom.document = $(document);
            
            dom.wrap = $wrap;
        	
            return dom;
        },
        
		/**********  @actionName  抖动效果     **************/
       	shake:(function () {
		    var fx = function (ontween, onend, duration) {
		        var startTime = + new Date;
		        var timer = setInterval(function () {
		            var runTime = + new Date - startTime;
		            var pre = runTime / duration;
		                
		            if (pre >= 1) {
		                clearInterval(timer);
		                onend(pre);
		            } else {
		                ontween(pre);
		            };
		        }, 13);
		    };
		    
		    var animate = function (elem, distance, duration) {
		        var quantity = arguments[3] || 6,
                    style = elem.style,
                    from = parseInt(style.marginLeft) || 0;
				
		        duration = duration / quantity;
		        
		        fx(function (pre) {
		            style.marginLeft = from + (distance - from) * pre + 'px';
		        }, function () {
		            if (quantity !== 0) {
		                animate(
		                    elem,
		                    quantity === 1 ? 0 : (distance / quantity - distance) * 1.3,
		                    duration,
		                    -- quantity
		                );
		            };
		        }, duration);
		    };
		    
		    return function () {
		        animate(this.dom.wrap[0], 20, 300);
		        return this;
		    };
		})(),
        
        /*****************************************************
		 * @actionName  获取当前对话框 dom
		 ****************************************************/
        getCurrentContent:function(){
        	return this.config.content;
        },
        
       	/*****************************************************
		 * @actionName  获取某对话框  
		 * @param   id	指定id获取对应的弹出框实例
		 ****************************************************/
        getIdByContent: function(id){
            var model = dialogPanel.contentCollection.get(id);
        	if(model) return model;
        	else return this;
        },
        
        /**********  @actionName  添加事件代理  **********/
        addEvent: function() {
			
            var _this = this, dom = this.dom;

            // 监听点击面板
            dom.wrap.bind('tap.dialogevent', _handleWrap);
            
            function _handleWrap(event) {
            	var target = event.target,name = target.getAttribute('name');
            	switch(name){
            		case 'dialogClose':
            			_this.close();
            			break;
            		default:
            			break;
            	}
            }
        },
		
        /**********  @actionName  卸载事件代理  **********/
        removeEvent: function() {
            this.dom.wrap.unbind('.dialogevent');
        }
        
    };
    
    /**********************************************
	 * @actionName  数组排序功能.
	 * @param array array 表示当前需要排序的数组对象
	 * 		  c string 表示排序的条件
	 * *******************************************/
    function _sort(array,c){
		array.sort(function(a,b){
			return Math.abs(a[c])<Math.abs(b[c])?1:Math.abs(a[c])==Math.abs(b[c])?0:-1
		});
		return array;
	}

	/**********  @actionName 默认配置  **********/
    dialogPanel.defaults = {
		
		// 添加弹出框ID
		id:null,
		
		// 弹出框背景透明度
		bgOpacity:null,
		
        // 消息内容
        content: null,

        // 对话框初始化后执行的函数
        initialize: null,

        // 对话框关闭前执行的函数
        beforeunload: null,

        // 对话框关闭后执行的函数
        afterunload:null,
        
        // 内容宽度
        width: 'auto',

        // 内容高度
        height: 'auto',

        // 皮肤名(多皮肤共存预留接口)
        skin: null,

        // 自动关闭时间
        time: null,

        // 是否支持Esc键关闭
        esc: true,

        // 是否支持对话框按钮自动聚焦
        focus: null,

        // 初始化后是否显示对话框
        visible: true,
        
        // 对话框关闭的时间
        delay: 300,

        // 是否锁屏
        lock: true,
        
        // 给面板添加样式
        type:null,

        // 是否固定定位
        fixed: true,
        
        // 对话框叠加高度值(重要：此值不能超过浏览器最大限制)
        zIndex: 1987

    };
    
	dialogPanel.fn.constructor.prototype = dialogPanel.prototype;

    $.dialog = dialogPanel;

});