(function($){

	// 定义常量类
	var CONSTANT = {
		TOUCHFLAG   : ('ontouchstart' in this),
		SINGLEDELAY : 250,
		LONGDELAY   : 2000,
		DOUBLEDELAY : 750
	};

	$.touch = {
		swipeOffset:10,
		tapOffset:5,
		moveOffset:5
	};

		// 定义私有变量
	var touch = {},
		// 定义判断是否是点击还是移动事件标志
		moveFlag = false,
		tapFlag = false,
		// 定义移动的距离差
		distanceX = distanceY = null,
		// 定义target对象
		targetTouch = targetTouchs = touchs = null,
		pageY = pageX = null,
		// 定义timer变量
		singleTapTimeout = longTapTimeout = null,
		// 定义事件类型
		events = {
			start  : CONSTANT.TOUCHFLAG ? 'touchstart'  : 'mousedown',
			move   : CONSTANT.TOUCHFLAG ? 'touchmove'   : 'mousemove',
			end    : CONSTANT.TOUCHFLAG ? 'touchend'    : 'mouseup',
			cancel : CONSTANT.TOUCHFLAG ? 'touchcancel' : ''
		};

	// 创建Touch类
	var Touch = function(){};

	// 共有静态属性和方法
	Touch._delta = null;

	Touch._dataNow = null;

	Touch._swipeDirection = function(x1, x2, y1, y2) {
		return Touch._crossDirection(x1,x2,y1,y2) == 'level' ? (x1 - x2 > 0 ? 'left' : 'right') : (y1 - y2 > 0 ? 'up' : 'down');
	};

	Touch._crossDirection = function(x1, x2, y1, y2){
		var xDelta = Math.abs(x1 - x2), yDelta = Math.abs(y1 - y2);
		return xDelta >= yDelta ? 'level' : 'vertical';
	};

	Touch._touchSwipe = function(){
		// 根据开始横纵坐标和完成横纵坐标,可以计算出左划,右划,上划,下划的方向事件
        Touch._triggerEvent(Touch._swipeDirection(touch.startX, touch.moveX, touch.startY, touch.moveY)+'swipe');
	};

	Touch._clearTimeout = function(timer){
		if (timer) clearTimeout(timer);
		timer = null;
	};

	Touch._triggerEvent = function(eventName){
		if(touch.$el)touch.$el.trigger($.Event(eventName));
		touch = {};
	};

	Touch.prototype.onTouchStart = function(event){

		if(event.button == 2) return false;

		// 获取所有的多点触控
		touchs = Array.prototype.slice.call(event.touches?event.touches:[event],0);
		// 不能让event事件中途发生变化,所以只能支持单点事件
		if(touchs.length > 1) return false;

		moveFlag = true;

		// 克隆触摸点的数组
		targetTouch = touchs[0];

		// 得到当前touch对象DOM
		touch.$el = $(targetTouch.target);

		// 抛出mousedown事件
		touch.$el.trigger($.Event('downtap',targetTouch),event);

		// 清空短时间点击事件
		singleTapTimeout && Touch._clearTimeout(singleTapTimeout);

		// 得到当前的时间
		Touch._dataNow = Date.now();
		// 得到当前时间差
		Touch._delta = Touch._dataNow - (touch.last || Touch._dataNow);
		// 得到时间差
		if (Touch._delta > 0 && Touch._delta <= CONSTANT.DOUBLEDELAY) touch.isDoubleTap = true;
		// 得到touch最后操作时间
		touch.last = Touch._dataNow;
		// 注册长时间事件
		longTapTimeout = setTimeout(Touch._triggerEvent, CONSTANT.LONGDELAY,'longtap');
		
		// 缓存当前touchstart的横纵坐标
		touch.startX = targetTouch.pageX;
		touch.startY = targetTouch.pageY;

		return false;
	};

	Touch.prototype.onTouchMove = function(event){
		if(!moveFlag) return false;

		// 清空长时间点击事件
		Touch._clearTimeout(longTapTimeout);

		if($.isEmptyObject(touch)) return false;

		// 获取所有的多点触控
		touchs = Array.prototype.slice.call(event.touches?event.touches:[event],0);

		// 不能支持多点转化
		if(targetTouch.target !== touchs[0].target) return false;

		// 克隆触摸点的数组
		targetTouch = touchs[0];

		touch.$el = $(targetTouch.target);

		pageX = targetTouch.pageX;

		pageY = targetTouch.pageY;

		// 缓存当前touchmove的横纵坐标
		if(pageX == touch.moveX && pageY == touch.moveY) return false;

		// 抛出mousemove事件
		touch.$el.trigger($.Event('movetap',targetTouch),event);

		touch.moveX = pageX;
		touch.moveY = pageY;

		// 抛出跟随滑动事件
		touch.$el.trigger($.Event('cross'),{
			clientX:touch.moveX - touch.startX,
			clientY:touch.moveY - touch.startY
		});

		return false;
	};

	Touch.prototype.onTouchEnd = function(event){
		moveFlag = false;

		// 清空长时间点击事件
		Touch._clearTimeout(longTapTimeout);

		if($.isEmptyObject(touch)) return false;

		touch.$el = touch.$el || $(targetTouch.target);

		// 抛出mouseup事件
		touch.$el.trigger($.Event('uptap'),event);

		if(touch.moveX >= 0 && touch.moveY >= 0){
			tapFlag = false;
			distanceX = Math.abs(touch.startX - touch.moveX);

			distanceY = Math.abs(touch.startY - touch.moveY);
		}else{
			tapFlag = true;
		}

		if(!tapFlag && (distanceX > $.touch.tapOffset || distanceY > $.touch.tapOffset)){
			// 触发手机滑动事件
			touch.$el.trigger($.Event('swipe'));

			// 当横或纵坐标大于30时,就触发移动事件
			if(distanceX > $.touch.swipeOffset || distanceY > $.touch.swipeOffset){
				// 触发手机长距离滑动事件
				touch.$el.trigger($.Event('longswipe'));
				// 根据开始横纵坐标和完成横纵坐标,可以计算出左划,右划,上划,下划的方向事件
				Touch._touchSwipe();
			}else{
				// 触发手机短距离滑动事件
				Touch._triggerEvent('singleswipe');
			}

		}else{
			// 触发手机点击事件
			touch.$el.trigger($.Event('tap'));

			// 如果是双击状态,就触发双击事件
			if (touch.isDoubleTap) {
				Touch._triggerEvent('doubletap');
			}else {
				singleTapTimeout = setTimeout(Touch._triggerEvent, CONSTANT.SINGLEDELAY,'singletap');
			}
		}
	};
	
	Touch.prototype.onTouchCancel = function(event){
		Touch._clearTimeout(longTapTimeout);
		Touch._clearTimeout(singleTapTimeout);
	    touch = {};
	};
	
	Touch.prototype.onTouchEvent = function(event){
		switch(event.type){
			case events.start:
				this.onTouchStart(event);
				break;
			case events.move:
				this.onTouchMove(event);
				break;
			case events.end:
				this.onTouchEnd(event);
				break;
			case events.cancel:
				this.onTouchCancel(event);
				break;
		}
	};

	$(document).ready(function(){

		var _this = new Touch();

		$(document).bind([events.start,events.move,events.end,events.cancel].join(' '),$.proxy(_this,'onTouchEvent'));
		
		$(window).bind('scroll', $.proxy(_this,'onTouchCancel'));

	});

	['downtap','movetap','uptap','swipe', 'singleswipe', 'longswipe', 'leftswipe', 'rightswipe', 'upswipe', 'downswipe','cross', 'tap', 'singletap', 'longtap', 'doubletap'].forEach(function(m){
		$.fn[m] = function(callback){ return this.bind(m, callback) };
	});

})(Zepto)