define(function(require, exports, module) {

	var winAlert = require('ui/dialog/alert/alert.js');

	require('common/storage.js');

	c360.storage.set('hasVisitChangeEmailPage', 'true');

	var PageView = Backbone.View.extend({

		el: document.getElementById('changeEmailPage'),

		template: _.template($('#page-template').html()),

		events: {
			'tap #checkbox': 'select',
			'tap #changeEmail': 'changeEmail',
			'tap #unchange': 'unchange',
			'focus #emailInput': 'changePosition',
			'blur #emailInput': 'changePosition'
		},

		changePosition: function(event) {
			if (!utils.isIOS()) {
				if (event.type == 'focus') {
					$('.content').animate({
						height: $('.content').height() + 100
					}, 250);
				} else if (event.type == 'blur') {
					$('.content').animate({
						height: $('.content').height() - 100
					}, 250);
				}
			}
		},

		initialize: function() {
			var _this = this;
			this.model = new Backbone.Model();
			window.cNative.getLoginUser(function(resp) {
				if (resp.error == 200) {
					if (resp.data.face == '' || resp.data.face == undefined) {
						_this.face = 'images/userbig.png';
					} else {
						_this.face = 'http://dn-c360.qbox.me/' + resp.data.face + '-95';
					}
					_this.model.set({
						nickname: resp.data.nickname,
						email: resp.email,
						face: _this.face
					});
					_this.render();
				}
			});
		},

		render: function() {
			this.$el.html(this.template(this.model.toJSON()));

			$('.content').height($(window).height() - 70);

			var img = new Image();
			img.src = this.face;
			img.onload = function() {
				$('#face').attr('src', img.src);
			};
		},

		select: function(event) {
			var target = $(event.target),
				status = target.attr('data-status');
			if (status == 'checked') {
				target.removeClass('chekced').addClass('default').attr('data-status', 'default');
			} else {
				target.removeClass('default').addClass('checked').attr('data-status', 'checked');
			}
		},

		unchange: function() {
			if ($('#checkbox').attr('data-status') == 'default') {
				c360.server.yahooNoPrompt(function(resp) {
					if (resp.status == 200) {
						window.cNative.getLoginUser(function(res) {
							res.data.changeEmail = 0; //设置下次不需要提醒更换邮箱
							res.expire = 82000;
							window.cNative.setLoginUser(res, function() {
								window.cNative.closeTopestView();
							});
						});
					} else {
						window.cNative.showBubbleMessage({
							msg: resp.message,
							duration: 2
						});
						window.cNative.closeTopestView();
					}
				});
			} else {
				window.cNative.closeTopestView();
			}
		},

		changeEmail: function() {
			var _this = this,
				email = $.trim($('#emailInput').val());
			if (email == '') {
				window.cNative.showBubbleMessage({
					msg: lang.haveNoEmail,
					duration: 2
				});
			} else if (!utils.checkEmail(email)) {
				window.cNative.showBubbleMessage({
					msg: lang.falseFormatEmail,
					duration: 2
				});
			} else if(email.toLowerCase().indexOf('@yahoo.com.cn') >=0 || email.toLowerCase().indexOf('@yahoo.cn') >=0){
				window.cNative.showBubbleMessage({
					msg: lang.notAllowedEmail,
					duration: 2
				});
			}else {
				c360.server.changeEmail({
					email: email
				}, function(res) {
					if (res.status == 200) {
						window.cNative.setLoginUser({
							email: res.data.userInfo.email,
							data: res.data.userInfo,
							pwd:res.data.userInfo.localkey,
							uid:res.data.userInfo.userId,
							expire:82000
						}, function(data) {
							if (data.error == 200) {
								_this.model.set({
									email: email
								});
								new SuccessView({
									model: _this.model
								});
							}
						});
					} else if (res.status == 402) {
						window.cNative.showBubbleMessage({
							msg: lang.changeEmailNotice,
							duration: 3
						});
						$('#emailInput').val('');
					} else {
						window.cNative.showBubbleMessage({
							msg: res.message,
							duration: 2
						});
					}
				});
			}
		}

	});

	var pageView = new PageView();

	var SuccessView = Backbone.View.extend({

		el: document.getElementById('changeEmailPage'),

		template: _.template($('#sucess-template').html()),

		events: {
			'tap #iKnow': 'iKnow'
		},

		initialize: function() {
			this.render();
		},

		render: function() {
			this.$el.html(this.template(this.model.toJSON()));
			var img = new Image();
			img.src = this.model.get('face');
			img.onload = function() {
				$('#face').attr('src', img.src);
			};
		},

		iKnow: function() {
			window.cNative.closeTopestView();
		}
	});


});