define(function(require, exports, module) {
	
	$.ns('c360.footer.draw');

	require('component/crop/crop');

	var FooterView = require('page/crop/footer');

	require('css/crop.css');

	require('common/storage');

	var CONSTANT = {
			PADDING:30
		},

		disabled = 'disabled',			// 定义点击失效class样式

		cropPage = Backbone.View.extend({

			el:document.body,

			// 定义裁切内容模版
			template:_.template($('#draw-page-template').html()),

			/**********************************************
			 * @actionName	初始化页面.
			 *********************************************/
			initialize:function(){

				this.model = new Backbone.Model();

				this.render();

				c360.footer.draw = new FooterView();

				this.initDom();

				this.initEventListener();
			},

			/**********************************************
			 * @actionName	渲染页面.
			 *********************************************/
			render:function(){
				this.$el.append(this.template(this.model.toJSON()));

				this.$el = $('#drawPage');

				this.el = this.$el[0];
			},

			/**********************************************
			 * @actionName	初始化DOM.
			 *********************************************/
			initDom:function(){
				this.$footerRightDom = this.$('.footer-right');
				this.$panelDom = this.$('#panel');
				this.$drawFooterDom = this.$('#drawFooter');
			},

			/**********************************************
			 * @actionName	显示裁切页面.
			 *********************************************/
			show:function(){
				$(document.body).addClass('crop-active');
				this.$el.show();
	            this.initImage();
			},

			/**********************************************
			 * @actionName	隐藏裁切页面.
			 *********************************************/
			hide:function(){
				$(document.body).removeClass('crop-active');
				this.$el.hide();
			},

			/**********************************************
			 * @actionName	初始化照片.
			 *********************************************/
			initImage:function(){
				
				this.$footerRightDom.addClass(disabled);

				c360.notification.loading(true,'Loading');

				this.image = document.createElement('img');

				this.$panelDom.empty().append(this.image);

				this.$imageDom = $(this.image);

				this.image.src = c360.storage.get('picUrl');

				this.image.onload = $.proxy(this,'onload');

				this.image.onerror = $.proxy(this,'onerror');
			},

			/**********************************************
			 * @actionName  图片加载失败.
			 * *******************************************/
			onerror:function(){
				c360.notification.loading(false,'Loading');
				c360.notification.bubble(lang.loadingImageError);
			},

			/**********************************************
			 * @actionName  图片加载成功.
			 * *******************************************/
			onload:function(){
				
				this.$footerRightDom.removeClass(disabled);

				c360.notification.loading(false,'Loading');

				this.imageWidth = this.image.width;

				this.imageHeight = this.image.height;

				var bodyWidth = $(document.body).width(),

					bodyHeight = $(document.body).height(),

					width = bodyWidth - CONSTANT.PADDING*2,

					height = bodyHeight - CONSTANT.PADDING*2 - this.$drawFooterDom.height()*2,

					rate = Math.max(1,this.imageWidth/width,this.imageHeight/height),

					imageWidth = this.imageWidth/rate,

					imageHeight = this.imageHeight/rate,

					photoSize = Math.min(imageWidth,imageHeight,100),

					pos = {
						width:imageWidth,
						height:imageHeight
					};

				this.rate = rate;

				// 这里必须做一个json对象的克隆技术
				this.$panelDom.show().css($.extend({},pos,{
					left:(bodyWidth - imageWidth)/2,
					top:(bodyHeight - imageHeight)/2
				}));

				this.$imageDom.css(pos).crop({
					minSize:[photoSize,photoSize],
	                bgOpacity: .6,
	                aspectRatio:1,
	                onSelect: $.proxy(this,'crop')
	            });

				this.$el.addClass('active');
			},

			/**********************************************
			 * @actionName  记录每次裁切后的坐标及大小.
			 * *******************************************/
			crop:function(event){
				if(event.w == 0 || event.h==0) return;
				this.photeData = {
					x:event.x,
					y:event.y,
					w:event.w,
					h:event.h,
					size:200
				}
			},

			/**********************************************
			 * @actionName  初始化页面事件.
			 * *******************************************/
			initEventListener:function(){
				c360.events.on('showcrop',this.show,this);
				c360.events.on('hidecrop',this.hide,this);
			}
		});


	return cropPage;
});