define(function(require, exports, module) {

	require('component/audio/nativeAudio');

	var CONSTANT = c360.constant.photo, //定义页面常量类

		AudioView = Backbone.View.extend({

			el:document.getElementById('photoAudio'),

			template:_.template($('#photo-audio-template').html()),

			eventNamespace:'.playerevent',

			ready:false,

			statusIndex:0,	// status表示音频加载状态(1:表示读取成功,2:表示加载成功)

			/**********************************************
			 * @actionName	注册页面事件.
			 *********************************************/
			events:{
				'click .on-audio':'handleAudio'
			},

			/**********************************************
			 * @actionName	初始化页面.
			 *********************************************/
			initialize:function(){
				// 得到照片面板对象
				this.photo = c360.container.photo;

				this.render();

				this.initEventListener();
			},

			/**********************************************
			 * @actionName	初始化DOM.
			 *********************************************/
			initDom:function(){
				this.$audioDom = this.$('.on-audio');
				if(this.$audioDom.length == 0){
					this.$audioDom = this.$('.off-audio');
				}
				this.$playerPanelDom = this.$('.player-panel');
			},

			/**********************************************
			 * @actionName	进入操作音频 功能.
			 *********************************************/
			handleAudio:function(){
				if(this.$audioDom.hasClass(CONSTANT.ACTIVE)){
					this.stop();
				}else{
					this.play();
				}
			},

			/**********************************************
			 * @actionName  设置音频播放器
		     * @param src string 表示音频播放器路径
			 * *******************************************/
			setMedia:function(src){
				if(this.photo.model.get('hasAudio') == 0) return;
				this.statusIndex++;
				var _this = this;
				if(this.ready){
					// audioFlag为了避免用户触发点击临界点,而用的标志位
					if(this.statusIndex == 2 && this.photo.audioFlag){
						this.$playerPanelDom.player('setMedia',src);
					}
				}else{
					this.$playerPanelDom.player({});
				}
				this.statusIndex = 1;
			},

			/**********************************************
			 * @actionName  播放音频 功能
			 * *******************************************/
			play:function(){
				if(this.$audioDom.hasClass(CONSTANT.ACTIVE)) return;
				if(this.photo.model.get('hasAudio') == 1){
					if($.audio.media){
						this.$playerPanelDom.player('play');
					}else{
						this.$playerPanelDom.player('setMedia',this.photo.model.get('audioSrc'));
					}
				}
			},

			/**********************************************
			 * @actionName  停止音频 功能
			 * *******************************************/
			stop:function(){
				if(this.photo.model.get('hasAudio') == 1){
					this.$playerPanelDom.player('stop');
				}
			},

			/**********************************************
			 * @actionName  内存释放
			 * *******************************************/
			release:function(){
				if($.audio.media) this.$playerPanelDom.player('release');
			},

			/**********************************************
			 * @actionName  渲染音频模块
			 * *******************************************/
			render: function() {
				var model = this.photo.model;

				this.$el.append(this.template(model.toJSON()));

				this.initDom();
			},

			/**********************************************
			 * @actionName  播放音频布局 功能
			 * *******************************************/
			playLayout:function(){
				if(this.photo.model.get('hasAudio') == 1){
					this.$audioDom.addClass(CONSTANT.ACTIVE);
				}else{
					this.$playerPanelDom.player('stop');
				}
			},

			/**********************************************
			 * @actionName  停止音频布局 功能
			 * *******************************************/
			stopLayout:function(){
				if(this.$audioDom.hasClass(CONSTANT.ACTIVE))
					this.$audioDom.removeClass(CONSTANT.ACTIVE);
			},

			/**********************************************
			 * @actionName  注册音频播放事件
			 * *******************************************/
			initEventListener:function(){
				var _this = this,

					eventObj = $.audio.event,

					model = this.photo.model.toJSON();

				this.$el.bind(eventObj.ready + this.eventNamespace, function(event) {
					_this.ready = true;
					_this.setMedia(model.audioSrc);
				});

				this.$el.bind(eventObj.play + this.eventNamespace, function(event) {
					_this.playLayout();
				});

				this.$el.bind(eventObj.ended + this.eventNamespace + ' ' +
							  eventObj.error + this.eventNamespace, function(event) {
					_this.stopLayout();
				});

			}
		});

	return AudioView;

});