define(function(require, exports, module) {

	var PhotoView = require('./photo'),

		MainView = require('./main'),

		loadingFlag = false,	// 记录loading的标志

		CONSTANT = c360.constant.photo, //定义页面常量类

		ContainerView = MainView.extend({

			photoFlag:false,			//  记录照片标志位

			albumFlag:false,			//  记录相册标志位

			audioFlag:true,				//  记录音符的标识

			animateFlag:null,			//  记录动画完成标志

			deleteFlag:null,			//  记录删除的标识

			el:document.getElementById('photoWrap'),

			pid:utils.getQuery('pid'), 	//	记录当前pid

			aid:utils.getQuery('aid'),	//	记录当前aid

			/**********************************************
			 * @actionName	初始化页面.
			 *********************************************/
			initialize:function(){
				c360.page.photo.$el = $('#photoPage');

				c360.header.photo.$el = $('#photoHeader');

				MainView.prototype.initialize.call(this);
			},

			/**********************************************
			 * @actionName	定义页面事件.
			 *********************************************/
			events:$.extend({
				'leftswipe .photo-img':'nextPhoto',
				'rightswipe .photo-img':'prevPhoto'
			}, MainView.prototype.events),

			/**********************************************
			 * @actionName 加载图片.
			 * @param id 照片ID
			 * *******************************************/
			loadPhoto:function(id){
				var model = this.collection.get(id);

				if(!model) return;

				var element = this.getPhotoDom(id),
					load = model.get('load');

				if(this.model.get('id') == id){
					// 加载成功关闭loading
					if(this.model.get('load') == 1){
						this.setPhotoSize(element,model);
						this.hideLoad();
					}else{
						this.showLoad();
					}
				}

				if(load == 0) return;

				// load为0时,表示正在加载这张照片
				// load为-1时,表示未加载这张照片
				// load为1时,表示加载完成这张照片
				// load为2时,表示加载失败这张照片
				if(load == 2 || load == -1){

					model.set('load',0);
					
					var image = new Image();

					image.id = id;

					image.model = model;

					image.src = model.get('data-url');

					image.onload = this.onload;

					image.onerror = this.onerror;

				}
			},

			/**********************************************
			 * @actionName 显示loading.
			 * *******************************************/
			showLoad:function(msg){
				if(c360.page.photo.$el.hasClass(CONSTANT.DISABLED)) return;
				if(loadingFlag) return;
				loadingFlag = true;
				notification.loading(true,msg || 'Loading');
			},

			/**********************************************
			 * @actionName  隐藏loading.
			 * *******************************************/
			hideLoad:function(msg){
				if(c360.page.photo.$el.hasClass(CONSTANT.DISABLED)) return;
				// 如果正在请求相册,并当前照片是第一张或最后一张时不关闭loading
				if(this.albumFlag){
					if(this.hasFirstModel() || this.hasLastModel()) return;
				}
				if(!loadingFlag) return;
				loadingFlag = false;
				notification.loading(false,msg || 'Loading');
			},

			/**********************************************
			 * @actionName  图片加载成功.
			 * *******************************************/
			onload:function(){
				var photo = c360.container.photo,
					id = this.id,
					model = this.model,
					element = photo.getPhotoDom(id);

				model.set('load',1);

				// 加载成功的照片必须存在,如果不存在会造成图片切换渲染问题
				if(element.length){
					photo.setPhotoUrl(element,model);

					photo.setPhotoSize(element,model);
				}
			},

			/**********************************************
			 * @actionName  图片加载失败.
			 * *******************************************/
			onerror:function(){
				var photo = c360.container.photo,
					id = this.id,
					model = this.model,
					pid = photo.model.get('id'),
					element = photo.getPhotoDom(id);

				model.set('load',2);

				// 加载成功的照片必须存在,如果不存在会造成图片切换渲染问题
				if(element.length){
					if(pid == id){
						if(photo.audioFlag){
							photo.audio.setMedia(photo.model.get('audioSrc'));
						}
						photo.hideLoad();
					}

					element.addClass('error');
				}
			},

			/**********************************************
			 * @actionName  设置model.
		     * @param id (int/string) (int:表示collection的位置得到model,string:表示id得到model)
			 * *******************************************/
			setModel:function(id){
				if(typeof id == 'number'){
					this.model = this.collection.at(id);
				}else{
					this.model = this.collection.get(id);
				}
			},

			/**********************************************
			 * @actionName  添加照片.
		     * @param model object 表示model模型对象
		     * @param collection object 表示model集合对象
		     * @param options json 表示触发事件时，传入的参数
			 * *******************************************/
			addPhoto:function(model,collection,options){
				model = this.setPhotoPos(model);

				this.$el.append((new PhotoView({model:model})).$el);
			},

			/**********************************************
			 * @actionName  添加相册.
			 * @param id string 表示相册ID
		     * @param collection object 表示model集合对象
		     * @param flag string 表示添加方向
			 * *******************************************/
			addAlbum:function(id,collection,flag){
				if(collection.length == 0) return;

				this.animateFlag = false;

				this.collection = collection;

				this.aid = id;

				this.index = 0;

				var data = this.cache.getData(id),
					_this = this,
					model1 = collection[flag](),
					model2 = element = null,
					id = model1.get('id'),
					length = collection.length,
					addPhoto = function(){
						if(flag == 'last'){
							_this.$el.prepend(element);
						}else{
							_this.$el.append(element);
						}
					},
					createPhoto = function(type){
						element = _this.resetPanel(model1,type);
						addPhoto();
						if(model2){
							element = _this.resetPanel(model2,type);
							addPhoto();
						}else{
							_this.$currentPanelDom[type]().remove();
						}
					};

				this.pre = data.pre;

				this.next = data.next;

				this.setPhotoCount(data.total);

				this.detail.model.set({
					total:data.total,
					name:data.name
				});

				if(length != 1){
					if(flag == 'last'){
						model2 = collection.at(length - 2);
					}else{
						model2 = collection.at(1);
					}
				}

				if(flag == 'last'){
					createPhoto('next');
				}else{
					createPhoto('prev');
				}

				this.$currentPanelDom.remove();

				this.layout(id,'');
			},

			/**********************************************
			 * @actionName  删除照片.
		     * @param model object 表示model模型对象
		     * @param collection object 表示model集合对象
		     * @param options json 表示触发事件时，传入的参数
			 * *******************************************/
			removePhoto:function(model,collection,options){
				var count = this.getPhotoCount() - 1,

					pid = model.get('id'),

					_this = this,

					length = this.collection.length,

					aid = this.aid,

					data = this.cache.getData(aid),

					pre = this.pre,

					next = this.next,

					callback = function(){
						this.setPhotoCount(count);
						this.syncData(pid,aid);
					};

				this.cache.setData(aid,$.extend(data,{
					total:count
				}));

				// 如果总数量为1的时候,如果是相册页面进入就直接返回上一个页面,如果是主页面进入就跳转下一个相册数据
				if(count == 0){
					callback.call(this);
					// 因为当前相册已经删除了,清空当前collection的数据
					this.cache.removeData(aid);
					this.cache.removeCollection(aid);
					if(next === '' || next === undefined){
						c360.backButtonFn();
					}else{
						// 因为当前相册已经删除了,所以需要重置上一个相册的next的ID值
						data = this.cache.getData(pre);
						if(data){
							this.cache.setData(pre,$.extend(data,{
								next:next
							}));
						}
						// 因为当前相册已经删除了,所以需要重置下一个相册的pre的ID值
						data = this.cache.getData(next);
						if(data){
							this.cache.setData(next,$.extend(data,{
								pre:pre
							}));
						}else{
							this.detail.model.set('index',0);
							// 当collection没有数据的时候,出现广告位图片并30秒后提示网络超时
							this.$currentPanelDom.remove();
							c360.page.photo.$el.addClass(CONSTANT.DISABLED + ' ' + CONSTANT.OFF);
							this.showLoad();
							this.loadTimer && clearTimeout(this.loadTimer);
							this.loadTimer = setTimeout(function(){
								clearInterval(_this.interTimer);
								_this.hideLoad();
								notification.bubble(lang.timeout);
							},CONSTANT.TIMEOUT);
							this.interTimer && clearInterval(this.interTimer);
							this.interTimer = setInterval(function(){
								data = _this.cache.getData(next);
								if(data){
									clearTimeout(_this.loadTimer);
									clearInterval(_this.interTimer);
									_this.hideLoad();
									c360.page.photo.$el.removeClass(CONSTANT.DISABLED + ' ' + CONSTANT.OFF);
									//_this.getAlbum(next,'first',lang.noNextPhoto,lang.nextAlbumLoading);
								}
							},CONSTANT.TIMEINTERVAL);
						}
						this.getAlbum(next,'first',lang.noNextPhoto,lang.nextAlbumLoading);
					}
				}else if(length == 0){
					callback.call(this);
					// 当collection没有数据的时候,出现广告位图片并30秒后提示网络超时
					this.$currentPanelDom.remove();
					c360.page.photo.$el.addClass(CONSTANT.DISABLED + ' ' + CONSTANT.OFF);
					this.showLoad();
					this.loadTimer && clearTimeout(this.loadTimer);
					this.loadTimer = setTimeout(function(){
						_this.hideLoad();
						notification.bubble(lang.timeout);
					},CONSTANT.TIMEOUT);
				}else {

					this.deleteFlag = true;

					var index = options.index,

						deletePhoto = function(){
							// 当index等于总数-1时或者等于当前collection总数时,都读取上一张照片
							if(index == count || index == length){
								// 这里索引回退是为了解决获取上一张照片
								_this.prevPhoto(_this.$currentPanelDom,true);
							}else{
								// 这里索引回退是为了解决获取下一张照片
								_this.nextPhoto(_this.$currentPanelDom,true);
							}
							callback.call(_this);
						};
					if(this.$currentPhotoDom.hasClass('isScale')){
						this.scaleEvent(this.$currentPhotoDom,function(){
							// 由于css3动画完成后需要一个0毫秒的延迟
							setTimeout(deletePhoto,0);
						},true);
					}else{
						deletePhoto();
					}

				}
			},

			/**********************************************
			 * @actionName  获取上一张照片.
		     * @param event object 表示事件event
			 * *******************************************/
			prevPhoto:function(event,flag){

				// 如果动态加载图片就屏蔽滑动功能
				if(c360.page.photo.$el.hasClass(CONSTANT.DISABLED) && flag === undefined) return;

				var target = event.currentTarget || event,
					$target = $(target).find('.single-photo');

				// 如果是全屏模式时,就不做照片切换功能
				if($target.hasClass('isScale')) return;

				if(this.animateFlag) return;
				this.animateFlag = true;

				// 当前第一张时,就还原当前的距离
				if(this.currentIndex == 0){
					this.getAlbum(this.pre,'last',lang.noPrevPhoto,lang.prevAlbumLoading);
				}else{
					// 翻上一张为临界点的时候,翻上一张需要特殊处理
					if(this.photoIndex == 0){
						this.getPhoto('first');
						return;
					}
					--this.photoIndex;
					// 根据当前缓存的distance距离,加上应该滑动的距离interval
					this.distanceLevel += this.interval;
					// 设置滑动的距离
					this.translatePhoto('first');
				}
			},

			/**********************************************
			 * @actionName  获取下一张照片.
		     * @param event object 表示事件event
			 * *******************************************/
			nextPhoto:function(event,flag){
				// 如果动态加载图片就屏蔽滑动功能
				if(c360.page.photo.$el.hasClass(CONSTANT.DISABLED) && flag === undefined) return;

				var target = event.currentTarget || event,
					$target = $(target).find('.single-photo');

				// 如果是全屏模式时,就不做照片切换功能
				if($target.hasClass('isScale')) return;

				if(this.animateFlag) return;
				this.animateFlag = true;

				// 当最后一张时,就还原当前的距离
				if(this.currentIndex == this.getPhotoCount() - 1){
					this.getAlbum(this.next,'first',lang.noNextPhoto,lang.nextAlbumLoading);
				}else{
					if(!this.deleteFlag) {
						// 翻下一张为临界点的时候,翻下一张需要特殊处理
						if(this.photoIndex == this.currentTotal - 1){
							this.getPhoto('last');
							return;
						}
						++this.photoIndex;
					}
					// 根据当前缓存的distance距离,减去应该滑动的距离interval
					this.distanceLevel -= this.interval;
					// 设置滑动的距离
					this.translatePhoto('last');
				}
			},

			/**********************************************
			 * @actionName  设置照片滑动的距离
			 * @param distance int 表示当前滑动的距离
			 * *******************************************/
			translatePhoto:function(direction){
				if(direction){
					// 关闭load图片timer
					this.hideLoad();
					this.loadCurrentTimer && clearTimeout(this.loadCurrentTimer);
					this.loadCurrentTimer = null;
				}

				var _this = this,

					callback = function(){
						clearTimeout(_this.animateTimer);
						// 检查并更新photo数据
						if(direction){
							_this.layout(_this.photoIndex,direction);
						}else{
							if(c360.page.photo.$el.hasClass(CONSTANT.DISABLED))
								c360.page.photo.$el.removeClass(CONSTANT.DISABLED);
						}
						setTimeout(function(){
							_this.animateFlag = false;
						},0);
					};

				// 在操作很快时,在IOS上CSS3动画可能不触发,所以为了回调的调用成功,如果不触发CSS3回调就300毫秒后自动触发
				this.$el.animate({
					translateX:this.distanceLevel+'px'
				},'fast',null,callback);

				this.animateTimer && clearTimeout(this.animateTimer);
				this.animateTimer = setTimeout(callback,CONSTANT.TIMELAYOUT);
			},

			/**********************************************
			 * @actionName  得到当前body的大小.
			 * *******************************************/
			getBodyPos:function(){
				return this.bodyPos = {
					width  : $(document.body).width(),
					height : $(document.body).height()
				};
			},

			/**********************************************
			 * @actionName  设置当前照片等比例缩放的大小.
			 * *******************************************/
			setPhotoPos:function(model){
				var panelPos = this.panelPos || this.getPanelPos(),
					bodyPos = this.bodyPos || this.getBodyPos(),
					photoPos = this.getPhotoPos(model),
					panelRate = Math.max(1,photoPos.width/panelPos.width,photoPos.height/panelPos.height),
					bodyRate = Math.max(1,photoPos.width/bodyPos.width,photoPos.height/bodyPos.height);
				
				model.set({
					pw:photoPos.width/panelRate,
					ph:photoPos.height/panelRate,
					bw:photoPos.width/bodyRate,
					bh:photoPos.height/bodyRate,
				});

				return model;
			},

			/**********************************************
			 * @actionName  得到当前照片的大小.
			 * *******************************************/
			getPhotoPos:function(model){
				return this.photoPos = {
					width  : model.get('w'),
					height : model.get('h')
				};
			},

			/**********************************************
			 * @actionName  设置当前照片的大小.
			 * *******************************************/
			setPhotoSize:function(element,model){
				var pid = this.model.get('id'),
					json = model.toJSON(),
					_this = this,
					scale = 1,
					time = 0,
					callback = null;

				// 如果不是当前照片就做padding的限制为了做动画有边距
				if(pid == json.id){
					this.hideLoad();
					callback = function(){
						clearTimeout(_this.audioTimer);
						_this.audioTimer = null;
						if(_this.audioFlag){
							_this.audio.setMedia(json.audioSrc);
						}
					}
					scale = json.bw/json.pw;
					// 如果当前设备是Google就不做缩放动画
					if(scale == 1 || cNative.deviceType == 'Galaxy Nexus'){
						setTimeout(callback,0);
					}else{
						time = 'fast';
						// 有可能缩放动画不触发回调所以需要手动触发
						this.audioTimer && clearTimeout(this.audioTimer);
						this.audioTimer = setTimeout(callback,CONSTANT.TIMELAYOUT);
					}
				}
				element.animate({
					scale:scale
				},time,null,function(){
					if($.isFunction(callback)){
						setTimeout(callback,0);
					}
				});
			},

			/**********************************************
			 * @actionName  设置面板位置.
			 * *******************************************/
			setPanelPos:function(){
				// 得到每次的步长
				this.interval = this.bodyPos.width;
				var index = 0,
					$currentPanelDom = this.$currentPanelDom;
				// 缓存当前滑动的距离
				// 有可能滑动太快CPU计算不过来会找不到,会出现黑屏
				if($currentPanelDom.length){
					index = $currentPanelDom.index();
				}else{
					if(this.photoIndex != 0){
						index = 1;
					}
				}
				this.distanceLevel = -index*this.interval;
				// 定义横纵坐标的位移
				this.distanceVertical = 0;
			},

			/**********************************************
			 * @actionName  得到当前面板的大小.
			 * *******************************************/
			getPanelPos:function(){
				var bodyPos = this.bodyPos || this.getBodyPos();
				return this.panelPos = {
					width  : bodyPos.width - 2*CONSTANT.PADDING,
					height : bodyPos.height
				};
			},

			/**********************************************
			 * @actionName  设置照片URL.
			 * *******************************************/
			setPhotoUrl:function(element,model){
				element.css({
					width:model.get('pw')+'px',
					height:model.get('ph')+'px'
				}).attr({
					src:model.get('data-url')
				});
			},

			/**********************************************
			 * @actionName  页面布局.
			 * *******************************************/
			layout:function(id,direction){
				// 销毁上一个内存
				this.releaseAudio(true,true);

				this.setModel(id);

				this.photoIndex = this.getPhotoIndex();

				// 设置当前页码
				this.setPhotoIndex();

				this.currentIndex = this.detail.model.get('index') - 1;

				var collection = this.collection,

					prevModel = nextModel = 

					nextLoad = prevLoad = 

					nextId = prevId = null,

					_this = this,

					id = this.model.get('id'),

					currentLoad = this.model.get('load'),

					hasAudio = this.model.get('hasAudio');

				this.currentTotal = collection.length;

				if(this.audio){
					if(hasAudio == 1){
						this.audio.$audioDom.removeClass('off-audio').addClass('on-audio');
					}else{
						this.audio.$audioDom.removeClass('on-audio').addClass('off-audio');
					}
				}

				if(this.deleteFlag){
					this.deleteFlag = false;
					this.$currentPanelDom.remove();
				}

				// 根据当前的model的索引计算出前后model对象
				if(this.photoIndex == 0){
					nextModel = collection.at(this.photoIndex+1);
				}else if(this.photoIndex == this.currentTotal - 1){
					prevModel = collection.at(this.photoIndex-1);
				}else{
					nextModel = collection.at(this.photoIndex+1);
					prevModel = collection.at(this.photoIndex-1);
				}

				// 根据前后方向动态改变DOM的位置
				if(direction == 'first' && prevModel){
					// 判断是否存在上一个数据
					this.$el.prepend(this.resetPanel(prevModel,'next'));
				}else if(direction == 'last' && nextModel){
					// 判断是否存在下一个数据
					this.$el.append(this.resetPanel(nextModel,'prev'));
				}

				if(currentLoad == 2 || currentLoad == -1){
					this.loadCurrentTimer = setTimeout($.proxy(this,'loadPhoto'),CONSTANT.LOADTIMER,id);
				}else{
					this.loadPhoto(id);
				}

				if(nextModel){
					nextLoad = nextModel.get('load');
					nextId = nextModel.get('id');
					if(nextLoad == 2 || nextLoad == -1){
						this.loadNextTimer = setTimeout($.proxy(this,'loadPhoto'),CONSTANT.THATTIMER,nextId);
					}else{
						this.loadPhoto(nextId);
					}
				}

				if(prevModel){
					prevLoad = prevModel.get('load');
					prevId = prevModel.get('id');
					if(prevLoad == 2 || prevLoad == -1){
						this.loadPrevTimer = setTimeout($.proxy(this,'loadPhoto'),CONSTANT.THATTIMER,prevId);
					}else{
						this.loadPhoto(prevId);
					}
				}
				// 设置当前panel的DOM
				this.$currentPanelDom = this.getPhotoPanelDom(id);

				// 设置当前photo的DOM
				this.$currentPhotoDom = this.$currentPanelDom.find('.single-photo');

				this.setPanelPos();
				// 根据距离定位当前wrap的位置
				this.$el.animate({
					translateX:this.distanceLevel+'px'
				},0);

				if(this.getPhotoCount() != this.currentTotal){
					if(this.photoIndex + CONSTANT.PHOTOINDEX >= this.currentTotal){
						if(direction == 'last' || direction == ''){
							this.doPhoto('last');
						}
					}else if(this.photoIndex <= CONSTANT.PHOTOINDEX){
						if(direction == 'first' || direction == ''){
							if(this.photoIndex != this.currentIndex){
								this.doPhoto('first');
							}
						}
					}
				}

				if(c360.page.photo.$el.hasClass(CONSTANT.DISABLED))
				c360.page.photo.$el.removeClass(CONSTANT.DISABLED);
			}

		});

	return ContainerView;

});