define(function(require, exports, module) {

	var CONSTANT = c360.constant.photo,

		Cache = function(){

			this.collections = {};

			this.data = {};

			/**********************************************
			 * @actionName  判断data是否存在
			 * @param key string 表示data对应的key
			 * *******************************************/
			this.hasData = function(key){
				if(key in this.data){
					return true;
				}else{
					return false;
				}
			};

			/**********************************************
			 * @actionName  设置data数据
			 * @param key string 表示data对应的key
			 * @param json json 表示存入data的数据
			 * *******************************************/
			this.setData = function(key,json){
				this.data[key] = json;
			};

			/**********************************************
			 * @actionName  得到data数据
			 * @param key string 表示data对应的key
			 * *******************************************/
			this.getData = function(key){
				return this.hasData(key) ? this.data[key] : null;
			};

			/**********************************************
			 * @actionName  删除data数据
			 * @param key string 表示data对应的key
			 * *******************************************/
			this.removeData = function(key){
				delete this.data[key];
			};

			/**********************************************
			 * @actionName  删除所有的data数据
			 * *******************************************/
			this.removesData = function(){
				this.data = {};
			};

			/**********************************************
			 * @actionName  判断collection是否存在
			 * @param key string 表示collection对应的key
			 * *******************************************/
			this.hasCollection = function(key){
				if(key in this.collections){
					return true;
				}else{
					return false;
				}
			};

			/**********************************************
			 * @actionName  设置collection数据
			 * @param key string 表示collection对应的key
			 * @param data array 表示存入collection的数据
			 * @param type string 表示存储方式,向前存入还是后存入
			 * *******************************************/
			this.setCollection = function(key,data,type){
				this.reset(data);

				var collection = null,
					i = data.length - 1;
					
				if(this.hasCollection(key)){
					collection = this.collections[key];
				}else{
					collection = this.collections[key] = new Backbone.Collection();
					collection.on('remove', c360.container.photo.removePhoto, c360.container.photo);
				}
				
				// 如果方向是first的话,就向前插入数据
				if(type == 'first'){
					for(;i >= 0;i--){
						collection.unshift(data[i],{silent:true});
					}
				}else{
					collection.add(data,{silent:true});
				}
			};

			/**********************************************
			 * @actionName  得到collection数据
			 * @param key string 表示collection对应的key
			 * *******************************************/
			this.getCollection = function(key){
				return this.hasCollection(key) ? this.collections[key] : null;
			};

			/**********************************************
			 * @actionName  删除collection数据
			 * @param key string 表示collection对应的key
			 * *******************************************/
			this.removeCollection = function(key){
				delete this.collections[key];
			};

			/**********************************************
			 * @actionName  删除所有的collection数据
			 * *******************************************/
			this.removesCollection = function(){
				this.collections = {};
			};

			/**********************************************
			 * @actionName  得到当前index
			 * @param key string 表示collection对应的key
			 * @param id string 表示照片的ID
			 * *******************************************/
			this.getIndex = function(key,id){
				var index = -1;
				if(this.hasCollection(key)){
					var collection = this.collections[key],
						model = collection.get(id);
					index = collection.indexOf(model);
				}
				return index;
			};

			/**********************************************
			 * @actionName  扩展并重新组装数据
			 * @param data array 表示存入collection的数据
			 * *******************************************/
			this.reset = function(data){
				var k = 0,
					array = data.slice(0),
					length = array.length,
					value = null;

				// 组装最新的照片数据
				for(;k < length;k++){
					value = array[k];
					value.id = value.id || value.pid;
					value.load = -1;
					value['data-url'] = CONSTANT.PHOTOURL.replace('{n}',value.key);
					value.audioSrc = CONSTANT.AUDIOURL + value.key + '?avthumb/mp3';
					if(!('hasAudio' in value)){
						value.hasAudio = 0;
					}
				}

				return array;
			};

		}

		return Cache;

});