define(function(require, exports, module) {

	var CONSTANT = c360.constant.photo, //定义页面常量类

		EventView = Backbone.View.extend({

		/**********************************************
		 * @actionName	定义页面事件.
		 *********************************************/
		events:{
			'downtap .photo-img':'defaultEvent',
			'uptap .photo-img':'defaultEvent',
			'movetap .photo-img':'defaultEvent',
			'cross .single-photo':'crossEvent',
			'swipe .single-photo':'dragEvent',
			'singletap .photo-img':'tapEvent',
			'doubletap .single-photo':'scaleEvent',
			'singleswipe .single-photo':'swipeEvent',
			'upswipe .single-photo':'swipeEvent',
			'downswipe .single-photo':'swipeEvent'
		},

		/**********************************************
		 * @actionName	阻止事件冒泡.
		 *********************************************/
		defaultEvent:function(event,e){
			utils.stopBubble(e);
		},

		/**********************************************
		 * @actionName  监听跟随滑动事件.
	     * @param event object 表示事件event
	     * @param data json 如下:{
			clientX:表示横向移动的坐标
			clientY:表示纵向移动的坐标
	     }
		 * *******************************************/
		crossEvent:function(event,data){

			// 如果动态加载图片就屏蔽滑动功能
			if(c360.page.photo.$el.hasClass(CONSTANT.DISABLED)) return;

			this.releaseAudio(false,false);

			this.loadNextTimer && clearTimeout(this.loadNextTimer);

			this.loadNextTimer = null;
				
			this.loadPrevTimer && clearTimeout(this.loadPrevTimer);

			this.loadPrevTimer = null;

			var target = $(event.currentTarget),
				pos = {},
				element = null,
				clientX = data.clientX;

			// 根据当前的model的索引计算出前后model对象
			if(clientX < 0){
				element = this.$currentPanelDom.next();
			}else{
				element = this.$currentPanelDom.prev();
			}

			if(element.length){
				element.find('.single-photo').animate({
					scale:1
				},0,null);
			}

			// 在根据当前滑动的水平distance距离,可以实现跟随滑动功能
			this.distanceX = this.distanceLevel + clientX;

			pos.translateX = this.distanceX +'px';

			// 判断当前是否是全屏模式
			if(target.hasClass('isScale')){
				// 在根据当前滑动的竖直distance距离,可以实现跟随滑动功能
				this.distanceY = this.distanceVertical + data.clientY;
				pos.translateY = this.distanceY +'px';
			}

			this.$el.animate(pos,0);
		},

		/**********************************************
		 * @actionName  实现全屏拖拽的范围.
	     * @param event object 表示事件event
		 * *******************************************/
		dragEvent:function(event){
			var target = $(event.currentTarget);
			if(target.hasClass('isScale')){
				var model = this.model,
					// 得到当前的索引index
					index = target.parent().index(),
					// 得到当前的位移
					distance = -index * this.interval,
					// 记录照片的溢出的距离,其实10是照片marginButtom值
					photoPos = {
						posX:(model.get('maxW') - this.bodyPos.width)/2,
						posY:(model.get('maxH') - this.bodyPos.height)/2
					};
					
				// 得到横坐标位置的位移
				this.distanceLevel = Math.min(distance + photoPos.posX, Math.max(distance - photoPos.posX,this.distanceX));
				// 得到纵坐标位置的位移
				this.distanceVertical = Math.min(photoPos.posY, Math.max(-photoPos.posY,this.distanceY));
				
				// 执行照片移动的动画
				this.$el.animate({
					translateX:this.distanceLevel+'px',
					translateY:this.distanceVertical+'px'
				},'fast',null);
			}
		},

		/**********************************************
		 * @actionName  短时间点击.
	     * @param event object 表示事件event
		 * *******************************************/
		tapEvent:function(event){
			c360.page.photo.$el.toggleClass(CONSTANT.ACTIVE);
		},

		/**********************************************
		 * @actionName  照片缩放功能.
	     * @param event object 表示事件event
	     * @param callback function 表示缩放动画完成后的回调函数
		 * *******************************************/
		scaleEvent:function(event,callback,flag){

			// 如果动态加载图片就屏蔽滑动功能
			if(c360.page.photo.$el.hasClass(CONSTANT.DISABLED) && flag === undefined) return;

			// 如果不是加载成功,就不让他做双击放大功能
			if(this.model.get('load') != 1) return;

			var element = c360.page.photo.$el,
				target = event.currentTarget?$(event.currentTarget):event,
				model = this.model,
				_this = this,
				rate = this.getRate(),
				time = 0,
				$photoElements = target.closest('.photo-img').siblings();

			if(element.hasClass(CONSTANT.ACTIVE)){
				flag = true;
				element.removeClass(CONSTANT.ACTIVE);
			}else {
				flag = false;
			}

			// 设置照片放大后的实际大小
			model.set({
				maxW:rate*model.get('pw'),
				maxH:rate*model.get('ph')
			});

			if(target.hasClass('isScale')){
				this.setPanelPos();
				this.scale = model.get('bw')/model.get('pw');
				var callbackFun = function(){
					$photoElements.removeClass(CONSTANT.OFF);
					$.isFunction(callback) && callback.call(_this);
				};
			}else{
				this.scale = rate;
				$photoElements.addClass(CONSTANT.OFF);
				var callbackFun = function(){};
			}

			var transitionCallback = function(){
				_this.scaleTimer && clearTimeout(_this.scaleTimer);
				_this.scaleTimer = setTimeout(function(){
					if(cNative.deviceType == 'Galaxy Nexus'){
						time = 0;
						setTimeout(callbackFun,0);
					}else{
						time = 'fast';
					}
					// 判断是否是全屏模式
					if(target.hasClass('isScale')){
						// 根据距离定位当前wrap的位置
						_this.$el.animate({
							translateX:_this.distanceLevel + 'px'
						},time);
					}

					target.animate({
						scale:_this.scale
					},time,null,callbackFun);

					target.toggleClass('isScale');

					c360.header.photo.$el[0].removeEventListener('webkitTransitionEnd', transitionCallback, false);
				},0);
			};

			// 判断是否是有头部显示,如果有需要监听header动画回调,如果没有就直接做缩放动画
			if(flag){
				c360.header.photo.$el[0].addEventListener('webkitTransitionEnd', transitionCallback, false);
			}else{
				transitionCallback();
			}

		},

		/**********************************************
		 * @actionName  短距离滑动事件.
	     * @param event object 表示事件event
		 * *******************************************/
		swipeEvent:function(event){
			var target = $(event.currentTarget);
			if(target.hasClass('isScale')) return;
			// 当触发的是短距离滑动事件时,就还原到以前的距离
			this.$el.animate({
				translateX:this.distanceLevel+'px'
			},'fast',null);
		}
	});

	return EventView;

})