define(function(require, exports, module) {
	var EventView = require('./event'),

		CONSTANT = c360.constant.photo, //定义页面常量类

		MainView = EventView.extend({

			/**********************************************
			 * @actionName	初始化页面.
			 *********************************************/
			initialize:function(){
				$.touch.swipeOffset = Math.floor($(document.body).width()/CONSTANT.SWIPEMULTIPLE);
			},

			/**********************************************
			 * @actionName  得到照片面板.
		     * @param id string 表示照片的id
			 * *******************************************/
			getPhotoPanelDom :function(id){
				return this.$('section[data-id="' + id +'"]');
			},

			/**********************************************
			 * @actionName  得到照片dom.
		     * @param id string 表示照片的id
			 * *******************************************/
			getPhotoDom :function(id){
				return this.getPhotoPanelDom(id).find('.single-photo');
			},

			/**********************************************
			 * @actionName  设置照片index功能.
			 * *******************************************/
			setPhotoIndex:function(){
				this.detail.model.set('index',Math.min(Math.max(1,this.photoIndex + this.index + 1),this.getPhotoCount()));
			},

			/**********************************************
			 * @actionName  得到照片准确的位置index
			 * *******************************************/
			getPhotoIndex : function(){
				return this.collection.indexOf(this.model);
			},

			/**********************************************
			 * @actionName  设置照片的数量
		     * @param count int 表示照片的数量
			 * *******************************************/
			setPhotoCount : function(count){
				this.detail.model.set('count',count);
			},

			/**********************************************
			 * @actionName  得到照片总数
			 * *******************************************/
			getPhotoCount : function(){
				var model = this.detail.model,
					count = model?model.get('count'):model.get('total');
				return count;
			},

			/**********************************************
			 * @actionName  计算浮点数的精确小数点的位数.
		     * @param num float 表示需要精确的数
		     * @param pos int 表示需要精确小数点的位置
			 * *******************************************/
			fomatFloat:function(num,pos){     
	             return Math.round(num*Math.pow(10, pos))/Math.pow(10, pos);     
	        },

			/**********************************************
			 * @actionName  更新照片数据.
			 * @param model object 表示model对象
		     * @param type string 表示获取上一个还是下一个DOM的标志
			 * *******************************************/
			resetPanel:function(model,type){
				var id = model.get('id'),
					element = this.getPhotoPanelDom(id);

				// 如果当前的model的DOM存在就直接返回,不存在就创建
				if(element.length == 0){
					var $photoElement = null,
						load = model.get('load');

					element = this.$currentPanelDom[type]();
					
					if(element.length == 0){
						element = this.$currentPanelDom.clone();
					}

					$photoElement = element.attr('data-id',id).find('.single-photo').removeAttr('style').removeClass('error').attr('src','images/gray.png');

					if(load == -1){
						// 如果状态为-1时,就做照片处理,算法优化
						model = this.setPhotoPos(model);
					}else if(load == 1){
						// 如果没有网络的情况下重新加载
						if(navigator.network && navigator.network.connection.type == 'none'){
							model.set('load',-1);
						}else{
							this.setPhotoUrl($photoElement,model);
							this.setPhotoSize($photoElement,model);
						}
					}else if(load == 2){
						$photoElement.addClass('error');
					}
				}

				return element;
			},

			/**********************************************
			 * @actionName  得到最大放大的比例.
			 * *******************************************/
			getRate : function(){

				var bodyPos = this.bodyPos || this.getBodyPos(),
					model = this.model,
					// 获取当前照片缩放的大小
					photoPos = {
						width:model.get('bw'),
						height:model.get('bh')
					},
					// 计算照片等比例缩放的比例大小
					rate = this.fomatFloat(Math.max(1,bodyPos.width/photoPos.width,bodyPos.height/photoPos.height),1)+0.5;

				return rate;
			},

			/**********************************************
			 * @actionName  获取相册数据
			 * @param id string 表示相册的ID
		     * @param flag string 表示获取上一个相册还是下一个相册的标志
		     * @param message1 string 表示前面没有照片或最后一张照片的提示信息
		     * @param message2 string 表示正在获取上一个相册或下一个相册的提示信息
			 * *******************************************/
			getAlbum:function(id,flag,message1,message2){

				if(id === '' || id === undefined){
					// 如果没有相册id就气泡提示
					notification.bubble(message1);
					// 返回滚动的位置
					this.translatePhoto();
					return;
				}

				// 根据相册ID,获取相册的collection,如果存在就不发送网络请求
				var collection = this.cache.getCollection(id);
				if(collection){

					var length = collection.length;

					if(length == 0){
						notification.bubble(message1);
						this.translatePhoto();
					}else{
						this.addAlbum(id,collection,flag);
					}
				}else{
					this.showLoad(message2);
					this.translatePhoto();
					this.doAlbum(id,flag);
				}
			},

			/**********************************************
			 * @actionName  判断是否是最后一个model
			 * *******************************************/
			hasLastModel:function(){
				var lastModel = this.collection.last();
				// 判断是否存在最后一个model,有可能照片删除完,没办法找到最后一个model
				if(lastModel){
					return (this.model.get('id') == lastModel.get('id'));
				}else{
					// 照片删除完,也算是最后一个model,因为删除完照片也需要自动跨下一个相册
					return true;
				}
			},

			/**********************************************
			 * @actionName  判断是否是第一个model
			 * *******************************************/
			hasFirstModel:function(){
				var firstModel = this.collection.first();
				// 判断是否存在第一个model,有可能照片删除完,没办法找到最后一个model
				if(firstModel){
					return (this.model.get('id') == firstModel.get('id'));
				}else{
					// 照片删除完,也算是第一个model,因为删除完照片也需要自动跨上一个相册
					return true;
				}
			},

			/**********************************************
			 * @actionName  请求相册数据
			 * @param id string 表示相册的ID
			 * @param flag string 表示获取上一个相册还是下一个相册的标志
			 * *******************************************/
			doAlbum : function(id,flag){
				if(this.albumFlag) return;

				this.albumFlag = true;

				var _this = this;

				c360.server.photoInfo({
					aid:id,
					dir:1
				},function(json){

					_this.albumFlag = false;

					_this.hideLoad();

					if(json.status == 200){

						var data = json.data,
							list = data.list,
							albumData = data.albumData,
							aid = albumData.aid;

						albumData.total = list.length;

						_this.cache.setData(aid,albumData);

						// 把相册数据存储相册数据
						_this.cache.setCollection(aid,list,'');

						var collection = _this.cache.getCollection(aid);

						// flag标志为last表示获取前一个相册,first表示获取后一个相册
						// 逻辑必须这么写,因为可能出现相册一张的情况
						if(flag == 'last'){
							if(_this.hasFirstModel()) {
								_this.addAlbum(aid,collection,flag);
							}
						}else{
							if(_this.hasLastModel()) {
								_this.addAlbum(aid,collection,flag);
							}
						}

					}else{
						notification.bubble(json.message);
					}
				});
			},

			/**********************************************
			 * @actionName  获取照片数据
		     * @param flag string 表示前翻还是后翻
			 * *******************************************/
			getPhoto:function(flag){
				notification.bubble(lang.dataloading);
				this.translatePhoto();
				this.doPhoto(flag);
			},

			/**********************************************
			 * @actionName  请求照片数据
		     * @param flag string 表示前翻还是后翻
			 * *******************************************/
			doPhoto:function(flag){
				if(this.photoFlag) return;

				this.photoFlag = true;

				var _this = this,
					collection = this.collection,
					model = collection[flag](),
					length = null;

				c360.server.photoInfo({
					aid:this.aid,
					startTime:model.get('dateTime'),
					dir:flag == 'first' ? -1 : 1,
					limit:CONSTANT.PHOTOLIMIT
				},function(json){

					_this.photoFlag = false;

					if(json.status == 200){
						var list = json.data.list,
							length = list.length,
							type = false,
							element = null;

						list.slice(0,Math.min(length,_this.getPhotoCount() - collection.length));

						// count数量保持不变,所以需要处理获取的数量不能超过count总数
						_this.cache.setCollection(_this.aid,list,flag);

						if(flag == 'last'){
							element = _this.swapPhoto(_this.photoIndex+1,'next','prev');
							if(element){
								type = true;
								_this.$el.append(element);
							}
						}else{
							_this.index -= list.length;
							_this.photoIndex += list.length;
							element = _this.swapPhoto(_this.photoIndex-1,'prev','next');
							if(element){
								type = true;
								_this.$el.prepend(element);
							}
						}

						_this.currentTotal = collection.length;

						if(type){
							_this.setPanelPos();

							// 根据距离定位当前wrap的位置
							_this.$el.animate({
								translateX:_this.distanceLevel+'px'
							},0);
						}
					}else{

						notification.bubble(json.message);

					}

				});
			},

			/**********************************************
			 * @actionName  交换照片
		     * @param index int 表示根据index获取指定的Model
		     * @param next string 表示获取下一个dom的标志
		     * @param prev string 表示获取上一个dom的标志
			 * *******************************************/
			swapPhoto:function(index,next,prev){
				var model = this.collection.models[index],
					element = null,
					$currentPanelDom = this.$currentPanelDom,
					nextElement = $currentPanelDom[next](),
					prevElement = $currentPanelDom[prev]()[prev]();
				// 判断当前dom的下一个dom是否存在,如果不存在就创建,如果存在就不创建
				if(model && nextElement.length == 0) {
					// 如果当前dom的上一个dom的上一个dom是否存在,如果不存在就创建,如果存在就把这个dom转化一下
					if(prevElement.length == 0){
						element = this.resetPanel(model,next);
					}else{
						element = prevElement.attr('data-id',model.get('id')).find('.single-photo').removeAttr('style').removeClass('error').attr('src','images/gray.png');
					}
				}
				return element;
			},

			/**********************************************
			 * @actionName  释放音频文件
			 * *******************************************/
			releaseAudio:function(flag,type){
				this.audioFlag = flag;
				if(this.audio) {
					$.audio.type = 'release';
					this.audio.stop();
					if(type)this.audio.release();
				}
			},

			/**********************************************
			 * @actionName  释放没用的变量和timer
			 * *******************************************/
			clear:function(){
				this.hideLoad();

				this.cache.removesData();

				this.cache.removesCollection();

				this.loadCurrentTimer && clearTimeout(this.loadCurrentTimer);

				this.loadCurrentTimer = null;

		        this.loadNextTimer && clearTimeout(this.loadNextTimer);

		        this.loadNextTimer = null;
						
				this.loadPrevTimer && clearTimeout(this.loadPrevTimer);

				this.loadPrevTimer = null

		        this.loadTimer && clearTimeout(this.loadTimer);

		        this.loadTimer = null;

		        this.releaseAudio(false,true);
			},

			/**********************************************
			 * @actionName  数据通信
		     * @param pid int 表示照片的ID
		     * @param aid int 表示相册的ID
			 * *******************************************/
			syncData:function(pid,aid){
	        	// 当第一个相册封面时,最多返回15张照片,
	        	// 反之最多返回7张照片
	        	var data = this.cache.getData(aid),
	        		count = this.getPhotoCount(),
	        		total = data.count - this.detail.model.get('total'),
	        		index = c360.storage.get('isFirstAlbum')=="1"?15:7,
	        		collection = this.collection,
	        		length = Math.min(count,index,collection.length),
	        		length = length%4 ? length : length - 1,
	        		covers = [],
	        		pids = [],
	        		i = 0;

	        	for(; i < length; i++){
	        		covers.push(collection.models[i].toJSON());
		        }

		        pids.push(pid);

				c360.events.trigger("syncAlbum",{
					aid:aid,
					pids:pids,
					count:count+total,
					covers:covers
				});
			}
		});

	return MainView;

})