define(function(require, exports, module) {

	$.ns('c360.page.draw', 'c360.page.user');

	require('common/Notification');
	require('css/loading.css');
	require('common/storage');
	require('route/scrop');

	var CropPage = require('page/crop/index');

	var nicknameView = new(Backbone.View.extend({

		el: document.getElementById('nicknamePage'),

		template: _.template($('#nickname-template').html()),

		events: {
			'tap .back': 'back',
			'keyup .edit-name-input': 'listenerStatus',
			'change .edit-name-input': 'listenerStatus',
			'tap .deleteName': 'clear',
			'tap #ensure': 'ensure'
		},

		initialize: function() {
			this.model = new Backbone.Model();

			c360.events.on('hidenick', this.hide, this);

			c360.events.on('shownick', this.show, this);
		},

		clear: function() {
			$('.deleteName').removeClass('hastext').addClass('notext');
			$('.edit-name-input').val('');
		},

		ensure: function() {
			var _this = this;
			var target = $('.edit-name-input'),
				text = $.trim($('.edit-name-input').val());
			utils.cutstr(text, 20, function(text) {
				target.val(text);
				/*window.cNative.showBubbleMessage({
					msg:lang.nicklength,
					duration:2
				});*/
			});
			if (text.length < 4) {
				window.cNative.showBubbleMessage({
					msg: lang.nicklength,
					duration: 2
				});
			} else if (text == c360.page.user.model.get('nickname')) {
				_this.back();
			} else {
				$('.loadingImg').show();
				$('.grayback').show();
				c360.server.updateInfo({
					nick: text
				}, function(data) {
					if (data.status == 200) {
						c360.page.user.model.set({
							nickname: data.nick
						});
						$('.edit-name-input').val(data.nick);
						window.userInfo.data.nickname = data.nick;
						window.cNative.setLoginUser(window.userInfo, function() {});
						_this.back();
					} else {
						window.cNative.showBubbleMessage({
							msg: data.message,
							duration: 3
						});
					}
					$('.loadingImg').hide();
					$('.grayback').hide();
				});
				//向服务器和本地发送消息 
			}
		},

		listenerStatus: function(event) {

			var target = $('.edit-name-input'),
				original = $('.edit-name-input').val(),
				originalLen = original.length,
				length = $.trim(original).length;
			if (length > 0) {
				$('.deleteName').removeClass('notext').addClass('hastext');
			} else {
				$('.deleteName').removeClass('hastext').addClass('notext');
			}
			if (event.keyCode == 8) return;
			utils.cutstr(target.val(), 20, function(text) {
				target.val(text);
				window.cNative.showBubbleMessage({
					msg: lang.nicklength,
					duration: 2
				});
			});
		},

		back: function() {
			location.href = '#';
		},

		hide: function() {
			var _this = this;
			this.$el.animate({
				left: '100%'
			}, 200, null, function() {
				_this.$el.hide();
			});
		},

		show: function() {
			this.render('show');
		},

		render: function(arg) {
			//alert(c360.page.user.model.get('nickname'));
			this.model.set({
				nickname: utils.HTMLEnCode(c360.page.user.model.get('nickname'))
			});
			this.$el.show().html(this.template(this.model.toJSON()));
			if (arg == 'show') {
				//$('.edit-name-input').focus();
				this.$el.animate({
					left: 0
				}, 200);
			}
		}
	}));

	var userView = Backbone.View.extend({

		el: document.getElementById('userSettingPage'),

		template: _.template($('#user-template').html()),

		events: {
			'tap #changeNickname': 'nicknamePageShow',
			'tap #changeAvatar': 'getPhoto',
			'tap .back': 'back',
			'tap #logout': 'logout'
		},

		initialize: function() {

			if (c360.database) {
				c360.database.initDB(null, null, true); //只是建立数据库
			}

			var _this = this;
			this.model = new Backbone.Model();
			this.model.on('change', this.render, this);

			window.cNative.getLoginUser(function(data) {
				if (data.error == 200 && data.uid != '') {
					_this.saveData(data);
					_this.model.set({
						nickname: data.data.nickname,
						urlc: data.data.face == '' ? './images/userbig.png' : 'https://dn-c360.qbox.me/' + data.data.face + '-72'
					});
				} else if (data.uid == '') {
					window.cNative.setLoginUser({
						email: '',
						uid: '',
						pwd: '',
						expire: 0
					}, function(data) {
						if (window.localStorage) {
							window.localStorage.setItem('database.isCreated', 'false');
						}
						window.cNative.gotoNativeView({
							viewId: 'cloudHomeView'
						});
					});
				} else {
					c360.notification.bubble(data.message);
				}
			});
			// var data = {};
			// data.data = {
			// 	nickname:312321,
			// 	urlc:'cb64df0d29526d1803330b59477b80ee'
			// };

			// _this.saveData(data);
			// _this.model.set({
			// 	nickname:data.data.nickname,
			// 	urlc:'https://dn-c360.qbox.me/'+data.data.face+'-72'
			// });

			c360.events.on('hidecrop hidenick', this.show, this);

			c360.events.on('showcrop shownick', this.hide, this);

		},

		logout: function() {
			var snsName = '';
			if(window.userInfo)
			{
				var data = window.userInfo.data;
				if(data.third.length != 0)
				{
					var dataLength = data.third.length;
					for(var key = 0; key < dataLength; key++)
					{
						if(data.third[key].isLogin == true)
						{
							snsName = data.third[0].name;
						}
					}
				}
				if(!snsName && data.email !=""){
					snsName = 'email';
				}
			}
			switch(snsName)
			{
				case 'sina':
					c360.statis.storageKey("cloudLogout_sina");
					break;
				case 'qq':
				case 'qzone':
					c360.statis.storageKey("cloudLogout_qq");
					break;
				case 'facebook':
					c360.statis.storageKey("cloudLogout_facebook");
					break;
				case 'twitter':
					c360.statis.storageKey("cloudLogout_twitter");
					break;
				case 'email':
					c360.statis.storageKey("cloudLogout_mail");
					break;
			}
			
			c360.storage.clear();
			if (c360.database) {
				c360.database.clearAllTables();
			}

			window.cNative.userLogout(function(resp) {
				if (resp.error == 200) {
					window.cNative.stopUploading(function(data) {
						if (data.error == 200) {
							window.cNative.showStatusMessage({
								text: '', //string   消息内容
								type: 1, //int   消息类型 0,等待 1,完成 2,错误
								duration: 2, //int  消息持续显示的时长 -1,不消失,单位:秒
								animation: false, //bool   是否显示翻页动画
								immediate: true, //bool 是否立即显示消息，如果是true，之前消息队列中所有消息会被清空
								position: -1, //int 显示在状态栏的水平相对位置,默认在200
								width: -1, //int   宽度，默认120
							});
						}
					});
					window.cNative.gotoNativeView({
						viewId: 'cloudHomeView',
						from: ''
					});
				} else {
					window.cNative.showBubbleMessage({
						msg: resp.message,
						duration: 2
					});
				}
			});
		},


		saveData: function(data) {
			window.userInfo = data;
			window.userInfo.email = data.data.email;
			window.userInfo.expire = 0;
			delete window.userInfo.msg;
			delete window.userInfo.error;
		},

		back: function() {
			window.cNative.gotoNativeView({
				viewId: 'returnSettingView',
				from: 'userSetting.html'
			});
		},

		show: function() {
			this.$el.show();
		},

		hide: function() {
			this.$el.hide();
		},

		getPhoto: function() {
			//this.onPhotoSuccess();
			navigator.camera.getPicture($.proxy(this, 'onPhotoSuccess'), $.proxy(this, 'onPhotoFail'), {
				quality: 70,
				sourceType: navigator.camera.PictureSourceType.PHOTOLIBRARY,
				destinationType: navigator.camera.DestinationType.FILE_URI
			});
		},

		onPhotoSuccess: function(imageUri) {
			c360.storage.set('picUrl', imageUri);
			if ($.isEmptyObject(c360.page.draw)) {
				c360.page.draw = new CropPage();
				$('#userSettingPage').hide();
			}
			location.href = '#page-crop/1';
		},

		onPhotoFail: function(message) {
			if(message.indexOf('cancelled') == -1) c360.notification.bubble(message);
		},

		nicknamePageShow: function() {
			location.href = '#page-nick/1';
		},

		render: function() {
			this.$el.html(this.template(this.model.toJSON()));
		}
	});

	c360.page.user = new userView();

});