define(function(require, exports, module) {
	$.ns('c360.share.at');

	require('ui/share/at.css');

	var CONSTANT = {
			PADDING:10
		},

		Base = require('ui/share/base');

	c360.share.at = new (Base.extend({

		/**********************************************
		 * @actionName	定义页面事件.
		 *********************************************/
		events:_.extend({
			'keyup .share-text':'sreach',
			'change .share-text':'sreach',
			'tap .share-at-text':'handleItem',
			'tap .share-at-cencel':'clear',
			'downtap .share-at-handle':'defaults',
			'movetap .share-at-handle':'defaults',
			'uptap .share-at-handle':'defaults',
			'cross .share-at-handle':'crossEvent',
			'swipe .share-at-handle':'swipeDragScope',
			'tap .back-panel':'toBack'
		},Base.prototype.events),

		collection:{
			sina:null,
			qq:null,
			qzone:null,
			email:null,
			twitter:null,
			facebook:null
		},

		/**********************************************
		 * @actionName	初始化页面.
		 *********************************************/
		initialize:function(){
			this.bodyHeight = $(window).height();
			Base.prototype.initialize.call(this);
			this.pageId = 'at';
			this.model.set({
				title:lang.selectAttention,
				back:lang.back,
				cencel:lang.cancel,
				value:lang.selectAttention,
				array:[]
			});
			// 添加fragment防止浏览器多次回流
			this.fragment = document.createDocumentFragment();
			this.model.on('change:array',this.setAtList,this);
		},

		/**********************************************
		 * @actionName	初始化DOM.
		 *********************************************/
		initDom:function(){
			this.$shareAtViewPortDom = this.$('.share-at-viewport');
			this.$shareAtHandleDom = this.$('.share-at-handle');
			this.$shareTextDom = this.$('.share-text');
		},

		// 定义分享@模版
		template:_.template("   <header class='header-color'>\
									<span class='back-panel'><span class='back-icon back-image'></span></span>\
									<span class='title-panel'><span class='title'><%= title %></span></span>\
								</header>\
								<div class='share-at-panel'>\
									<div class='share-at-content'>\
										<span class='share-at-input'>\
											<span class='share-at-holder'>\
												<input disabled='disabled' class='share-text' placeholder='<%= value %>' >\
											</span>\
										</span>\
										<span class='share-at-btn'>\
											<span class='share-at-cencel'><%= cencel %></span>\
										</span>\
									</div>\
								</div>\
								<div class='share-at-viewport'>\
									<div class='share-at-list'>\
										<div class='share-at-handle'>\
											<%for( var i = 0,len = array.length;i<len;i++){%>\
												<span class='share-at-text'><%= array[i] %></span>\
									    	<%}%>\
										</div>\
									</div>\
								</div>"),

		/**********************************************
		 * @actionName	设置at列表.
		 *********************************************/
		setAtList:function(model,array){

			this.$shareAtHandleDom.empty().removeAttr('style');
			this.clearScope(this.$shareAtHandleDom);
			var element = null;
			for(var i = 0,len = array.length; i < len; i++){
				element = document.createElement('span');
				element.textContent = array[i];
				element.className = 'share-at-text';
				// 把每一个element对象放进fragment里面去
				this.fragment.appendChild(element);
			}
			// 防止回流先把碎片装在到fragment里面，然后再统一打包放到DOM里面
			this.$shareAtHandleDom.append(this.fragment);
			this.setPanelPos();
		},

		/**********************************************
		 * @actionName  显示分享详情.
	     * @param flag int 表示方向(-1:表示分享@滑出,1:表示分享@滑入)
		 * *******************************************/
		show:function(flag){

			this.model.set({
				shareName:null
			},{silent:true});

			var code = c360.share.detail.model.get('code');

			if(document.getElementById('shareat')
			&& this.$el.hasClass(code)){
				setTimeout($.proxy(this,'animate'),0,0,true);
			}else{
				var info = c360.SNS.get(code),
					_this = this;
					
				this.searchArray = info.atUsers;

				this.model.set({
					array:this.searchArray,
					code:code
				},{silent:true});

				this.render();

				if(!this.collection[code]){
					c360.server.shareAt({
						siteCode:code,
						uid:info.uid
					},function(json){
						if(json.status == 200){
							_this.collection[code] = json.data;
						}else{
							notification.bubble(json.message);
						}
					});
				}

			}
		},

		/**********************************************
		 * @actionName  屏蔽所有输入框事件.
		 * *******************************************/
		offDisable:function(){
			if(this.$shareTextDom) this.$shareTextDom.attr('disabled','disabled');
		},

		/**********************************************
		 * @actionName  打开所有输入框事件.
		 * *******************************************/
		onDisable:function(){
			this.$shareTextDom.removeAttr('disabled');
		},

		/**********************************************
		 * @actionName  返回上一级页面.
		 * *******************************************/
		toBack:function(event){
			this.closeKeyboard();
			location.href = '#share-detail/'+this.model.get("code")+'/-1';
		},

		/**********************************************
		 * @actionName  隐藏分享@.
		 * *******************************************/
		hide:function(flag){
			this.animate($(document.body).width(),false);
		},

		/**********************************************
		 * @actionName  清空@列表.
		 * *******************************************/
		clear:function(){
			this.closeKeyboard();
			this.searchArray = [];
			this.model.set({
				array:this.searchArray
			});
			this.$shareTextDom.val('');
		},

		/**********************************************
		 * @actionName  关闭IOS的键盘.
		 * *******************************************/
		closeKeyboard:function(){
			this.$shareTextDom.blur();
			setTimeout(function(){
				window.scrollTo(0,document.body.scrollTop);
			},0);
		},

		/**********************************************
		 * @actionName  点击@列表.
		 * *******************************************/
		handleItem:function(event){
			var target = $(event.currentTarget);
			this.model.set('shareName',target.text()+' ');
			this.toBack();
		},

		/**********************************************
		 * @actionName  搜索@列表.
		 * param value string 表示当前输入的值
		 * param text string 表示匹配的值
		 * *******************************************/
		sreachEach:function(value, text){
		    var valueLen = value.length,
		    	textLen = text.length,
		    	compare = null,
		    	i = 0,
		    	index = -1;
		    
		    if(valueLen <= textLen){
				for(; i <= textLen - valueLen + 1; i++){
		            compare = text.substring(i, i + valueLen);
		            // 不区分大小写
		            if(compare.toLowerCase() == value.toLowerCase()){   
		                index = i;
		                break;
		            }
				}
		    }

		    return index;
		},

		/**********************************************
		 * @actionName  搜索@列表.
		 * *******************************************/
		sreach:function(event){
			var target = $(event.currentTarget),
				val = target.val(),
				pos = 0,
				codeArray = this.collection[this.model.get('code')],
				i = 0;

			if(!codeArray) return;

			var len = codeArray.length;

			this.searchArray = [];

			if(val != '') {

				for(;i < len; i++){
					pos = this.sreachEach(val,codeArray[i]);
					if(pos>=0){
		            	this.searchArray.push(codeArray[i]);
		            }
				}

			}

			this.model.set({
				array:this.searchArray
			});
		},

		/**********************************************
		 * @actionName  设置当前列表面板高度.
		 * *******************************************/
		setPanelPos:function(){
			if(this.searchArray.length == 0){
				this.$shareAtViewPortDom.removeClass('active');
			}else{
				this.$shareAtViewPortDom.addClass('active');
			}
			this.panelDom = this.$shareAtViewPortDom.height(Math.min(this.bodyHeight - this.$shareAtViewPortDom.offset().top - CONSTANT.PADDING*2,this.$('.share-at-handle').height()));
		},

		/**********************************************
		 * @actionName  动画完成后的回调函数.
		 * *******************************************/
		animateComplete:function(){
			Base.prototype.animateComplete.call(this);
			this.initDom();
			this.onDisable();
			this.setPanelPos();
			c360.events.trigger('hidesharedetail:event',1);
		},

		/**********************************************
		 * @actionName  初始化页面事件.
		 * *******************************************/
		initEventListener:function(){
			Base.prototype.initEventListener.call(this);
			c360.events.on('showshareat:event',$.proxy(this,'show'));
			c360.events.on('hideshareat:event',$.proxy(this,'hide'));
		}
	}));
});