define(function(require, exports, module) {

	var Base = Backbone.View.extend({

		tagName:'section',	// 指定HTML元素类型

		pageId:null,		// 定义页面ID

		distanceVertical:0, // 定义初始垂直位移

		events:{},			// 定义页面事件

		panelDom :$(document.body),

		/**********************************************
		 * @actionName	初始化页面.
		 *********************************************/
		initialize:function(){
			this.model = new Backbone.Model();
			this.initEventListener();
		},

		/**********************************************
		 * @actionName	阻止默认事件.
		 *********************************************/
		defaults:function(event,e){
			e.preventDefault();
		},

		/**********************************************
		 * @actionName  监听跟随滑动事件.
	     * @param event object 表示事件event
	     * @param data json 如下:{
			clientX:表示横向移动的坐标
			clientY:表示纵向移动的坐标
	     }
		 * *******************************************/
		crossEvent:function(event,data){

			var target = $(event.currentTarget);

			if(this.panelDom.height() >= target.height()) return;

			var pos = {};

			// 在根据当前滑动的竖直distance距离,可以实现跟随滑动功能
			this.distanceY = this.distanceVertical + data.clientY;
			pos.translateY = this.distanceY +'px';

			target.animate(pos,0);
		},

		/**********************************************
		 * @actionName  清空当前位置.
	     * @param element object 表示当前DOM
		 * *******************************************/
		clearScope:function(element){
			this.distanceVertical = 0;
			element.animate({translateY:'0px'},0);
		},

		/**********************************************
		 * @actionName  实现全屏拖拽的范围.
	     * @param event object 表示事件event
		 * *******************************************/
		swipeDragScope:function(event){

			var target = $(event.currentTarget),
				panelHeight = this.panelDom.height();

			if(panelHeight >= target.height()) return;

			// 得到纵坐标位置的位移
			this.distanceVertical = Math.min(0, Math.max(panelHeight - target.height(),this.distanceY));
			
			// 执行照片移动的动画
			target.animate({
				translateY:this.distanceVertical+'px'
			},'duration','linear');
		},

		/**********************************************
		 * @actionName  设置模版引擎.
		 * *******************************************/
		setTemplate:function(){
			this.$el.empty().append(this.template(this.model.toJSON()));
		},

		/**********************************************
		 * @actionName  渲染页面.
		 * *******************************************/
		render:function(){

			if(!document.getElementById('share'+this.pageId)){
				$(document.body).append(this.$el.attr('id','share'+this.pageId));
			}

			this.setTemplate();

			var code = this.model.get('code');

			this.$el.removeClass().addClass('share-page share-'+this.pageId+'-page'+(code?' '+code:''));

			this.animate(0,true);
		},

		/**********************************************
		 * @actionName  动画函数.
		 * @param step int 表示动画时间
		 * @param type boolean 表示滑入还是滑出(true:表示滑入,false:表示滑出)
		 * *******************************************/
		animate:function(step,type){
			var _this = this;

			if(this.pageId == 'list'){
				this.animateComplete();
			}else{
				this.$el.show().animate({
					translateX:step + 'px'
				},'duration','linear',function(){
					if(type) _this.animateComplete();
					else _this.$el.hide();
				});
			}
		},

		/**********************************************
		 * @actionName  动画完成后的回调函数.
		 * *******************************************/
		animateComplete:function(){

		},

		/**********************************************
		 * @actionName  初始化页面事件.
		 * *******************************************/
		initEventListener:function(){
			
		}
	});

	return Base;
});