define(function(require, exports, module) {
	$.ns('c360.share.detail');

	require('ui/share/at');

	require('ui/share/mask');

	require('ui/share/detail.css');

	//定义页面常量类
	var CONSTANT = {
			EMAIL:lang.emailShare,	// 定义email分享标题
			EMAILMAX:5				// 定义email最大个数
		},

		alertDialog = require('ui/dialog/alert/alert'),

		disabled = 'disabled',			// 定义点击失效class样式

		Base = require('ui/share/base'),

		loading = (function(){
			var value = '';
			if(lang.language == 'zh_CN'
				|| lang.language == 'zh_TW'){
				value = lang.sharing;
			}else{
				value = 'loading';
			}
			return value;
		})(),

		// 定义分享sns模版
		SNSTemplate = _.template("  <header class='header-color'>\
										<span class='back-panel'><span class='back-icon back-image'></span></span>\
										<span class='title-panel'><span class='title'><%= title %></span></span>\
										<span class='share-panel'>\
											<div class='share-btn-color share-icon btn-share'><%= share %></div>\
										</span>\
									</header>\
									<div class='sns-panel'>\
										<div class='sns-textarea-panel'>\
											<textarea disabled='disabled' class='sns-textarea' placeholder='<%= shareplaceholder %>'><%= shareplaceholder %></textarea>\
										</div>\
										<div class='sns-operate'>\
											<span class=<%= at==1?'sns-at-on':'sns-at-off' %>><div class='sns-at'></div></span>\
											<span class='sns-font'><%= currentsize %>/<%= fontsize %></span>\
										</div>\
										<div class='sns-code'><%= codeName %></div>\
									</div>\
									<div class='sns-image-panel'>\
										<%for( var i in key ){%>\
											<div class='sns-image-content sns-image-rotate<%= i %>'>\
												<img class='sns-photo' src='images/gray.png' data-url='<%= key[i] %>'/>\
											</div>\
								    	<%}%>\
								    	<span class='sns-image-tip'><%= count %></span>\
									</div>"),

		// 定义分享email模版
		emailTemplate = _.template("<header class='header-color'>\
										<span class='back-panel'><span class='back-icon back-image'></span></span>\
										<span class='title-panel'><span class='title'><%= title %></span></span>\
										<span class='share-panel'>\
											<div class='share-btn-color share-icon btn-share'><%= share %></div>\
										</span>\
									</header>\
									<div class='email-tip'>\
										<div class='email-title'>\
											<span class='email-text'><div class='email-desc'><%= text %></div><div class='email-num'><%= photo %></div></span>\
										</div>\
									</div>\
									<div class='email-content'>\
										<span class='email-list'>\
											<span class='email-input'>\
												<span class='email-input-title'><%= addresstitle %>:</span>\
												<input disabled='disabled' placeholder='<%= addressplaceholder %>' class='email-address'>\
											</span>\
											<span class='email-input'>\
												<span class='email-input-title'><%= captiontitle %>:</span>\
												<input disabled='disabled' value=<%= emailValue %> class='email-value' placeholder='<%= captionplaceholder %>'>\
											</span>\
										</span>\
										<span class='email-textarea'>\
											<span class='email-input-title'><%= texttitle %>:</span>\
											<textarea disabled='disabled' placeholder='<%= textplaceholder %>' class='email-container'></textarea>\
										</span>\
									</div>");

	c360.share.detail = new (Base.extend({

		/**********************************************
		 * @actionName	定义页面事件.
		 *********************************************/
		events:_.extend({
			'tap .btn-share':'doShare',
			'tap .sns-at-on':'toAt',
			'tap .sns-image-panel':'crop',
			'keyup .sns-textarea':'checkTextarea',
			'change .sns-textarea':'checkTextarea',
			'keyup .email-value':'checkInput',
			'change .email-value':'checkInput',
			'tap .back-panel':'toBack'
		},Base.prototype.events),

		/**********************************************
		 * @actionName	初始化页面.
		 *********************************************/
		initialize:function(){
			Base.prototype.initialize.call(this);
			this.pageId = 'detail';
			this.model.on('change:currentsize',this.setFontSize,this);
			this.model.on('change:text',this.setText,this);
			this.model.on('change:photo',this.setPhotoNum,this);
			this.model.on('change:count',this.setCount,this);
			this.model.on('change:title',this.setTitle,this);
		},

		/**********************************************
		 * @actionName	初始化DOM.
		 *********************************************/
		initDom:function(){
			this.$emailAddressDom = this.$('.email-address');
			this.$emailValueDom = this.$('.email-value');
			this.$snsFontDom = this.$('.sns-font');
			this.$snsImageTipDom = this.$('.sns-image-tip');
			this.$emailTextDom = this.$('.email-text');
			this.$emailNumDom = this.$('.email-num');
			this.$emailContainerDom = this.$('.email-container');
			this.$shareTitleDom = this.$('.title');
			this.$snsTextareaDom = this.$('.sns-textarea');
			this.$snsImageContentDom = this.$('.sns-image-content');
		},

		/**********************************************
		 * @actionName	检查input输入合法性.
		 *********************************************/
		checkInput:function(event){
			var target = $(event.currentTarget);
			// 处理最大输入30个字符
			utils.cutstr(target.val(),30,function(text){
				target.val(text);
				notification.bubble(lang.over30);
			});
		},

		/**********************************************
		 * @actionName	验证文本框输入合法性.
		 *********************************************/
		checkTextarea:function(event){
			var target = event.currentTarget?$(event.currentTarget):event,
				val = target.val(),
				code = this.model.get('code'),
				placeholder = this.$snsTextareaDom.attr('placeholder'),
				length = placeholder.length,
				check = false,
				message = null;
			// 判断当前站点是否是twitter,如果是twitter是验证100个字符最大限制,而且其他的都是100个字最大限制
			if(code == 'twitter'){
				if(val != ''){
					length = utils.substr(val);
				}
				// 截取最大字符不超过100
				utils.cutstr(val,100,function(text){
					target.val(text);
					message = lang.over100;
					notification.bubble(message);
					check = true;
				});
			}else{
				if(val != ''){
					length = Math.min(val.length,100)
				}
				// 截取最大字符不超过100
				if(val.length > 100){
					message = lang.length100;
					notification.bubble(message);
					check = true;
				}
				target.val(val.substring(0,length));
			}
			// 获取当前输入的字符
			this.model.set('currentsize',length);
			return check;
		},

		/**********************************************
		 * @actionName  返回上一级页面.
		 * *******************************************/
		toBack:function(code){
			if(typeof(code) === 'object' || this.model.get('code') == code){
				this.closeKeyboard();
				location.href = '#/-1';
			}
		},

		/**********************************************
		 * @actionName  进入@功能页面.
		 * *******************************************/
		toAt:function(){
			if(this.$el.hasClass(disabled)) return;
			location.href = '#share-at/1';
		},

		/**********************************************
		 * @actionName	设置当前输入的字符数量.
		 *********************************************/
		setFontSize:function(model,index){
			this.$snsFontDom.text(Math.min(index,100)+'/'+100);
		},

		/**********************************************
		 * @actionName	设置照片总数量.
		 *********************************************/
		setCount:function(){
			this.$snsImageTipDom.text(this.model.get('count'));
		},

		/**********************************************
		 * @actionName	设置输入框email的值.
		 *********************************************/
		setText:function(){
			this.$emailTextDom.text(this.model.get('text'));
			this.$emailValueDom.val(this.model.get('emailValue'));
		},

		/**********************************************
		 * @actionName	设置照片张数.
		 *********************************************/
		setPhotoNum:function(){
			this.$emailNumDom.text(this.model.get('photo'));
		},

		/**********************************************
		 * @actionName	设置分享标题.
		 *********************************************/
		setTitle:function(){
			this.$shareTitleDom.text(this.model.get('title'));
		},

		/**********************************************
		 * @actionName	定义拼图接口.
		 *********************************************/
		crop:function(){
			if(parseInt(this.model.get('count'),10) > 1){
				this.closeKeyboard();
				c360.share.mask.show();
			}
		},

		/**********************************************
		 * @actionName	定义模版变量.
		 *********************************************/
		template:null,

		/**********************************************
		 * @actionName  显示分享详情.
		 * @param code string 表示当前站点代号
	     * @param flag int 表示方向(-1:表示分享详情滑出,1:表示分享详情滑入)
		 * *******************************************/
		show:function(code,flag){
			notification.loading(false,'Loading');
			this.shareData = c360.storage.get("shareData");
			this.themeName = this.shareData.name;
			var modelData = {},
				count = this.shareData.count || this.shareData.pids.length;

			if(code == 'email'){
				modelData = {
					text:lang.toEmail.replace('{n}',this.themeName),
					photo:'('+count+lang.numPhoto+')',
					cancel:lang.cancel,
					addresstitle:lang.addressee,
					addressplaceholder:lang.enterAddress,
					captiontitle:lang.captionTitle,
					captionplaceholder:lang.addShareDesc,
					texttitle:lang.textTitle,
					title:CONSTANT.EMAIL,
					textplaceholder:lang.enterEmailText,
					emailValue:this.themeName
				};
			}else{
				var photos = this.shareData.photos,
					photo = null,
					array = [],
					i = 0,
					key = 'http://dn-c360.qbox.me/{n}-135',
					src = null,
					shareTheme = lang.cameraShare.replace('{n}',this.themeName);

				for(; i < 3;i++){
					photo = photos[i];
					if(photo){
						src = key.replace('{n}',photo.key);
					}else{
						src = 'images/gray_.png';
					}
					array.push(src);
				}

				modelData = {
					codeName:lang.toSns.replace('{n}',this.getShareName(code)),
					shareplaceholder:shareTheme,
					currentsize:shareTheme.length,
					fontsize:100,
					title:c360.SNS.get(code).nickname,
					count:count,
					key:array
				};

			}

			this.model.set($.extend(modelData,{
				code:code,
				share:lang.share,
				at:c360.SNS.get(code).at,
				back:lang.back
			}),{silent:true});

			if(flag == -1) {
				var value = c360.share.at.model.get("shareName"),
					val = this.$snsTextareaDom.val();

				if(value){
					var str = val+'@'+value;
					this.model.set('currentsize',str.length);
					this.$snsTextareaDom.val(str);
				}
				this.onDisable();
				this.$el.show();
				c360.events.trigger('hideshareat:event',-1);
			}else{
				if(document.getElementById('sharedetail')
				&& $('#sharedetail').hasClass(code)){
					setTimeout($.proxy(this,'animate'),0,0,true);
				}else{
					if(code == 'email'){
						this.template = emailTemplate;
					}else{
						this.template = SNSTemplate;
					}
					this.render();
				}
			}
		},

		/**********************************************
		 * @actionName  隐藏分享详情.
		 * *******************************************/
		hide:function(flag){
			if(c360.share.mask) c360.share.mask.hide();
			if(flag == -1){
				this.animate($(document.body).width(),false);
			}else{
				this.$el.hide();
			}
		},

		/**********************************************
		 * @actionName  设置照片位置.
		 * *******************************************/
		setPhotoPos:function(){
			var imageContent = this.$snsImageContentDom;

			if(imageContent.length){
				var	photoIcon = null,
					_this = this,
					key = this.model.get('key');

				imageContent.each(function(i,dom){
					photoIcon = $(this).find('.sns-photo');
					// 加载分享图片
					_this.loadImage(photoIcon,key[i]);
				});
			}
		},

		/**********************************************
		 * @actionName  异步加载图片.
		 * @param dom element dom元素
		 * @param src string 图片路径
		 * *******************************************/
		loadImage:function(dom,src){
			var image = new Image(),
				_this = this;

			image.onload = function(){
				dom.attr('src',this.src);
			};

			image.onerror = function(){
				dom.addClass('error');
			};

			image.src = src;
		},

		/**********************************************
		 * @actionName  屏蔽所有输入框事件.
		 * *******************************************/
		offDisable:function(){
			var code = this.model.get('code');
			if(code === undefined) return;
			if(code == 'email'){
				this.$emailValueDom.attr('disabled','disabled');
				this.$emailAddressDom.attr('disabled','disabled');
				this.$emailContainerDom.attr('disabled','disabled');
			}else{
				this.$snsTextareaDom.attr('disabled','disabled');
			}
		},

		/**********************************************
		 * @actionName  打开所有输入框事件.
		 * *******************************************/
		onDisable:function(){
			var code = this.model.get('code');
			if(code == 'email'){
				this.$emailValueDom.removeAttr('disabled');
				this.$emailAddressDom.removeAttr('disabled');
				this.$emailContainerDom.removeAttr('disabled');
			}else{
				this.$snsTextareaDom.removeAttr('disabled');
			}
		},

		/**********************************************
		 * @actionName  关闭IOS的键盘.
		 * *******************************************/
		closeKeyboard:function(){
			var code = this.model.get('code');
			if(code == 'email'){
				this.$emailValueDom.blur();
				this.$emailAddressDom.blur();
				this.$emailContainerDom.blur();
			}else{
				this.$snsTextareaDom.blur();
			}
			setTimeout(function(){
				window.scrollTo(0,document.body.scrollTop);
			},0);
		},

		/**********************************************
		 * @actionName  发送分享请求.
		 * *******************************************/
		doShare:function(event){
			var target = $(event.currentTarget);
			
			if(target.hasClass(disabled))return;

			var code = this.model.get('code'),
				_this = this;
				
			if(code == 'email'){
				var emailVal = this.$emailAddressDom.val(),
					emailArray = [],
					emailDom = this.$emailValueDom,
					emailContainerDom = this.$emailContainerDom,
					emails = [];
				// 根据';'解析email个数,并验证email合法性,不区分全角圆角
				if(emailVal.match(/;|；/)){
					if(emailVal.indexOf(';') != -1){
						emailArray = emailVal.split(';');
					}else{
						emailArray = emailVal.split('；');
					}
					for(var i = 0 ,len = emailArray.length;i < len;i++ ){
						if(utils.checkEmail(emailArray[i])){
							emails.push(emailArray[i]);
						}
					}
				}else{
					if(utils.checkEmail(emailVal)){
						emails.push(emailVal);
					}
				}
				
				// 验证email个数
				if(emails.length == 0) {
					notification.bubble(lang.emailNoCheck);
				} else if(emails.length > CONSTANT.EMAILMAX){ // 最大email数量为5
					notification.bubble(lang.EmailAddressMax);
				} else if(emailDom.val() == ''){ // 验证邮件主题是否为空
					notification.bubble(lang.shareNameNoNull);
				} else{

					target.addClass(disabled);

					this.$el.addClass(disabled);

					this.offDisable();

					notification.loading(true,loading);

					c360.server.shareEmails({
						aids:this.shareData.aids,
						pids:this.shareData.pids,
						emails:emails,
						title:emailDom.val(),
						content:emailContainerDom.val(),
						themeName:this.themeName
					},function(json){
						notification.loading(false,loading);
						if(json.status == 200){
							_this.$emailAddressDom.val('');
							emailContainerDom.val('');
							emailDom.val(_this.themeName);
							notification.bubble(lang.emailShareSuccess);
							_this.toBack('email');
						}else{
							_this.onDisable();
							notification.bubble(lang.emailShareError);
						}
						target.removeClass(disabled);
						_this.$el.removeClass(disabled);
					});
				}
			}else{
				// 验证分享信息输入的长度
				if(this.checkTextarea(this.$snsTextareaDom)) {
					target.removeClass(disabled);
					return;
				}

				var info = c360.SNS.get(code),
					textarea = this.$snsTextareaDom,
					value = textarea.val(),
					placeholder = textarea.attr('placeholder'),
					title = (value == '') ? placeholder : value,
					_this = this,
					val = '',
					siteCode = '';

				target.addClass(disabled);

				this.$el.addClass(disabled);

				this.offDisable();
				
				notification.loading(true,loading);
				c360.server.sharePhotos({
					aids:this.shareData.aids,
					pids:this.shareData.pids,
					title:title,
					sideCode:code,
					token:info.token,
					uid:info.uid,
					themeName:this.themeName
				},function(json){
					notification.loading(false,loading);
					if(json.data) siteCode = json.data.siteCode;
					if(siteCode){
						val = _this.getShareName(siteCode);
					}
					if(json.status == 200){
						textarea.val(placeholder);
						c360.share.mask.hide();
						notification.bubble(val+lang.shareSuccess);
						_this.toBack(siteCode);
					}else{
						// 如果expire等于1时,就表示绑定失效
						if(json.data && json.data.expire == 1){
							alertDialog.render({
								title:lang.errorTip,
								content:lang.bindErrorTip.replace('{n}',val),
								certain:lang.back
							},function(){
								this.close();
								_this.shareError(siteCode);
							},function(){
								_this.shareError(siteCode);
							});
						}else{
							_this.onDisable();
							notification.bubble(val+lang.shareError);
						}
					}
					target.removeClass(disabled);
					_this.$el.removeClass(disabled);
				});
			}
		},

		/**********************************************
		 * @actionName  得到当前分享名称.
		 * @param code string 表示当前站点的代号
		 * *******************************************/
		getShareName:function(code){
			var val = '';
			switch(code){
				case 'qzone':
					val = lang.qzone;
					break;
				case 'qq':
					val = lang.qq;
					break;
				case 'sina':
					val = lang.sina;
					break;
				case 'renren':
					val = lang.renren;
					break;
				case 'twitter':
					val = lang.twitter;
					break;
				case 'facebook':
					val = lang.facebook;
					break;
			}
			return val;
		},

		/**********************************************
		 * @actionName  分享失败的回调函数.
		 * @param code string 表示当前站点的代号
		 * *******************************************/
		shareError:function(code){
			c360.SNS.info[code].binding = 0;

			++c360.SNS.unbindLen;
			--c360.SNS.bindLen;

			c360.share.list.setShareList();

			this.toBack(code);
		},

		/**********************************************
		 * @actionName  动画完成后的回调函数.
		 * *******************************************/
		animateComplete:function(){
			Base.prototype.animateComplete.call(this);
			this.initDom();
			this.onDisable();
			this.setPhotoPos();
			c360.events.trigger('hidesharelist:event',1);
		},

		/**********************************************
		 * @actionName  初始化页面事件.
		 * *******************************************/
		initEventListener:function(){
			Base.prototype.initEventListener.call(this);
			c360.events.on('showsharedetail:event',$.proxy(this,'show'));
			c360.events.on('hidesharedetail:event',$.proxy(this,'hide'));
		}
	}));
});