define(function(require, exports, module) {
	$.ns('c360.share.email');

	require('ui/share/base.css');

	require('ui/share/email.css');

	require('common/storage');

	require('route/backButton');

	c360.backButtonFn = function(){
		//关闭loading
        cNative.displayLoadingIcon({show:false,msg:"Loading",modal:false});

		cNative.closeTopestView();
	}

	//定义页面常量类
	var CONSTANT = {
			EMAIL:lang.emailShare,	// 定义email分享标题
			EMAILMAX:5				// 定义email最大个数
		},

		disabled = 'disabled',			// 定义点击失效class样式

		notification = require('common/Notification'),

		loading = (function(){
			var value = '';
			if(lang.language == 'zh_CN'
				|| lang.language == 'zh_TW'){
				value = lang.sharing;
			}else{
				value = 'loading';
			}
			return value;
		})(),

		// 定义分享email模版
		emailTemplate = _.template("<header class='header-color'>\
										<span class='back-panel'><span class='back-icon back-image'></span></span>\
										<span class='title-panel'><span class='title'><%= title %></span></span>\
										<span class='share-panel'>\
											<div class='share-btn-color share-icon btn-share'><%= share %></div>\
										</span>\
									</header>\
									<div class='email-tip'>\
										<div class='email-title'>\
											<span class='email-text'><div class='email-desc'><%= text %></div><div class='email-num'><%= photo %></div></span>\
										</div>\
									</div>\
									<div class='email-content'>\
										<span class='email-list'>\
											<span class='email-input'>\
												<span class='email-input-title'><%= addresstitle %>:</span>\
												<input disabled='disabled' placeholder='<%= addressplaceholder %>' class='email-address'>\
											</span>\
											<span class='email-input'>\
												<span class='email-input-title'><%= captiontitle %>:</span>\
												<input disabled='disabled' value=<%= emailValue %> class='email-value' placeholder='<%= captionplaceholder %>'>\
											</span>\
										</span>\
										<span class='email-textarea'>\
											<span class='email-input-title'><%= texttitle %>:</span>\
											<textarea disabled='disabled' placeholder='<%= textplaceholder %>' class='email-container'></textarea>\
										</span>\
									</div>");

	c360.share.email = new (Backbone.View.extend({

		/**********************************************
		 * @actionName	定义页面事件.
		 *********************************************/
		events:_.extend({
			'tap .btn-share':'doShare',
			'keyup .email-value':'checkInput',
			'change .email-value':'checkInput',
			'tap .back-panel':'toBack'
		}),

		/**********************************************
		 * @actionName	初始化页面.
		 *********************************************/
		initialize:function(){
			this.model = new Backbone.Model();
			this.model.on('change:text',this.setText,this);
			this.model.on('change:photo',this.setPhotoNum,this);
			this.model.on('change:title',this.setTitle,this);
			this.show();
		},

		/**********************************************
		 * @actionName	初始化DOM.
		 *********************************************/
		initDom:function(){
			this.$emailAddressDom = this.$('.email-address');
			this.$emailValueDom = this.$('.email-value');
			this.$emailTextDom = this.$('.email-text');
			this.$emailNumDom = this.$('.email-num');
			this.$emailContainerDom = this.$('.email-container');
			this.$shareTitleDom = this.$('.title');
		},

		/**********************************************
		 * @actionName	检查input输入合法性.
		 *********************************************/
		checkInput:function(event){
			var target = $(event.currentTarget);
			// 处理最大输入30个字符
			utils.cutstr(target.val(),30,function(text){
				target.val(text);
				notification.bubble(lang.over30);
			});
		},

		/**********************************************
		 * @actionName  返回上一级页面.
		 * *******************************************/
		toBack:function(){
			this.closeKeyboard();
			c360.backButtonFn();
		},

		/**********************************************
		 * @actionName	设置输入框email的值.
		 *********************************************/
		setText:function(){
			this.$emailTextDom.text(this.model.get('text'));
			this.$emailValueDom.val(this.model.get('emailValue'));
		},

		/**********************************************
		 * @actionName	设置照片张数.
		 *********************************************/
		setPhotoNum:function(){
			this.$emailNumDom.text(this.model.get('photo'));
		},

		/**********************************************
		 * @actionName	设置分享标题.
		 *********************************************/
		setTitle:function(){
			this.$shareTitleDom.text(this.model.get('title'));
		},

		/**********************************************
		 * @actionName	定义模版变量.
		 *********************************************/
		template:null,

		/**********************************************
		 * @actionName  显示分享详情.
	     * @param flag int 表示方向(-1:表示分享详情滑出,1:表示分享详情滑入)
		 * *******************************************/
		show:function(){
			this.shareData = c360.storage.get("shareData");
			this.themeName = this.shareData.name;
			var count = this.shareData.count || this.shareData.pids.length;
			this.model.set({
				text:lang.toEmail.replace('{n}',this.themeName),
				photo:'('+count+lang.numPhoto+')',
				cancel:lang.cancel,
				addresstitle:lang.addressee,
				addressplaceholder:lang.enterAddress,
				captiontitle:lang.captionTitle,
				captionplaceholder:lang.addShareDesc,
				texttitle:lang.textTitle,
				title:CONSTANT.EMAIL,
				textplaceholder:lang.enterEmailText,
				emailValue:this.themeName,
				share:lang.share,
				back:lang.back
			},{silent:true});

			this.template = emailTemplate;

			this.render();
		},

		/**********************************************
		 * @actionName  隐藏分享详情.
		 * *******************************************/
		hide:function(flag){
			cNative.closeTopestView();
		},

		/**********************************************
		 * @actionName  屏蔽所有输入框事件.
		 * *******************************************/
		offDisable:function(){
			this.$emailValueDom.attr('disabled','disabled');
			this.$emailAddressDom.attr('disabled','disabled');
			this.$emailContainerDom.attr('disabled','disabled');
		},

		/**********************************************
		 * @actionName  打开所有输入框事件.
		 * *******************************************/
		onDisable:function(){
			this.$emailValueDom.removeAttr('disabled');
			this.$emailAddressDom.removeAttr('disabled');
			this.$emailContainerDom.removeAttr('disabled');
		},

		/**********************************************
		 * @actionName  关闭IOS的键盘.
		 * *******************************************/
		closeKeyboard:function(){
			this.$emailValueDom.blur();
			this.$emailAddressDom.blur();
			this.$emailContainerDom.blur();
			setTimeout(function(){
				window.scrollTo(0,document.body.scrollTop);
			},0);
		},

		/**********************************************
		 * @actionName  发送分享请求.
		 * *******************************************/
		doShare:function(event){
			var target = $(event.currentTarget);
			
			if(target.hasClass(disabled))return;

			var _this = this,
				emailVal = this.$emailAddressDom.val(),
				emailArray = [],
				emailDom = this.$emailValueDom,
				emailContainerDom = this.$emailContainerDom,
				emails = [];
			// 根据';'解析email个数,并验证email合法性,不区分全角圆角
			if(emailVal.match(/;|；/)){
				if(emailVal.indexOf(';') != -1){
					emailArray = emailVal.split(';');
				}else{
					emailArray = emailVal.split('；');
				}
				for(var i = 0 ,len = emailArray.length;i < len;i++ ){
					if(utils.checkEmail(emailArray[i])){
						emails.push(emailArray[i]);
					}
				}
			}else{
				if(utils.checkEmail(emailVal)){
					emails.push(emailVal);
				}
			}
			
			// 验证email个数
			if(emails.length == 0) {
				notification.bubble(lang.emailNoCheck);
			} else if(emails.length > CONSTANT.EMAILMAX){ // 最大email数量为5
				notification.bubble(lang.EmailAddressMax);
			} else if(emailDom.val() == ''){ // 验证邮件主题是否为空
				notification.bubble(lang.shareNameNoNull);
			} else{

				target.addClass(disabled);

				this.$el.addClass(disabled);

				this.offDisable();

				notification.loading(true,loading);

				c360.server.shareEmails({
					aids:this.shareData.aids,
					pids:this.shareData.pids,
					emails:emails,
					title:emailDom.val(),
					content:emailContainerDom.val(),
					themeName:this.themeName
				},function(json){
					notification.loading(false,loading);
					if(json.status == 200){
						_this.$emailAddressDom.val('');
						emailContainerDom.val('');
						emailDom.val(_this.themeName);
						notification.bubble(lang.emailShareSuccess);
						_this.toBack();
					}else{
						_this.onDisable();
						notification.bubble(lang.emailShareError);
					}
					target.removeClass(disabled);
					_this.$el.removeClass(disabled);
				});
			}
		},

		/**********************************************
		 * @actionName  渲染页面.
		 * *******************************************/
		render:function(){

			if(!document.getElementById('shareEmail')){
				$(document.body).append(this.$el.attr('id','shareEmail'));
			}

			this.$el.removeClass().addClass('share-page share-email');

			this.setTemplate();
		},

		/**********************************************
		 * @actionName  设置模版引擎.
		 * *******************************************/
		setTemplate:function(){
			this.$el.empty().append(this.template(this.model.toJSON()));
			this.initDom();
			this.onDisable();
		}

	}));
});