define(function(require, exports, module) {
	$.ns('c360.share.mask');

	require('ui/share/mask.css');

	require('common/storage');

	//定义页面常量类
	var CONSTANT = {
			PHOTONUM:8,	// 定义拼图模版显示最大数量
			PADDING:10	// 面板的padding值
		},

		Base = require('ui/share/base');

	c360.share.mask = new (Base.extend({
		className:'share-mask-page',

		/**********************************************
		 * @actionName	定义页面事件.
		 *********************************************/
		events:_.extend({
			'tap':'hide',
			'downtap .share-mask-page':'defaults',
			'movetap .share-mask-page':'defaults',
			'uptap .share-mask-page':'defaults',
			'downtap .share-mask-list':'defaults',
			'movetap .share-mask-list':'defaults',
			'uptap .share-mask-list':'defaults',
			'cross .share-mask-list':'crossEvent',
			'swipe .share-mask-list':'swipeDragScope'
		},Base.prototype.events),

		/**********************************************
		 * @actionName	初始化页面.
		 *********************************************/
		initialize:function(){
			Base.prototype.initialize.call(this);
			this.pageId = 'mask';
			this.windowW = $(window).width();
		},

		// 标识拼图显示状态
		isShow:false,

		/**********************************************
		 * @actionName	初始化DOM.
		 *********************************************/
		initDom:function(){
			this.$shareMaskItemDom = this.$('.share-mask-item');
			this.$shareMaskListDom = this.$('.share-mask-list');
		},

		// 定义拼图模版
		template:_.template("   <div class='share-mask-list'>\
									<%for( var i = 0,len = photos.length;i<len;i++){%>\
										<span class='share-mask-item' data-height=<%= photos[i].h %> >\
											<img class='share-mask-image' src='images/gray.png' data-url='<%= photos[i].key %>'/>\
							    		</span>\
							    	<%}%>\
								</div>"),

		/**********************************************
		 * @actionName  显示拼图页面.
		 * *******************************************/
		show:function(){

			this.isShow = true;

			this.panelDom = $(window);

			if(document.getElementById('sharemask')){
				this.$el.show();
				this.animateComplete();
			}else{
				var photos = c360.storage.get("shareData").photos,
					array = [],
					i = 0,
					len = Math.min(photos.length,CONSTANT.PHOTONUM),
					width = this.windowW/2 - 2 * CONSTANT.PADDING,height = 0,
					rate = null,
					headerHeight = 0,
					footerHeight = 0;

				var lang = utils.getLanguage();
				lang = lang.substring(lang.length-2,lang.length);
				for(;i < len;i++){
					rate = width/photos[i].w,
					height = photos[i].h*rate;
					if(i == 0){
						headerHeight = 40*width/440;
						array.push({
							key:'images/share_photos_header_'+lang+'.jpg',
							h:headerHeight
						});
					}
					array.push({
						key:'http://dn-c360.qbox.me/'+photos[i].key+'-small',
						h:height
					});
					if(i == len - 1){
						footerHeight = 147*width/440;
						array.push({
							key:'images/share_photos_footer_'+lang+'.jpg',
							h:footerHeight
						});
					}
				}

				this.model.set('photos',array);

				this.render();
			}
			
		},

		/**********************************************
		 * @actionName  隐藏拼图模版.
		 * *******************************************/
		hide:function(){
			this.isShow = false;
			this.$el.hide();
			c360.events.trigger("hidemask");
		},

		/**********************************************
		 * @actionName  设置照片位置.
		 * *******************************************/
		setPhotoPos:function(){
			var imageContent = this.$shareMaskItemDom,
				flag = false;

			if(imageContent.length){
				var	_this = this;
				imageContent.each(function(i,dom){
					// 异步加载图片
					if(!$(this).hasClass('success')) {
						flag = true;
						_this.loadImage($(this));
					}
				});
			}

			if(flag){
				this.$shareMaskListDom.animate({
					translateY:'0px'
				},0,'linear');
			}

		},

		/**********************************************
		 * @actionName  异步加载图片.
		 * @param dom element dom元素
		 * @param src string 图片路径
		 * *******************************************/
		loadImage:function(dom){
			var image = new Image(),
				photoDom = dom.find('.share-mask-image'),
				src = photoDom.attr('data-url');

			image.onload = function(){
				dom.css('height',dom.attr('data-height')+'px');
				dom.removeClass('error').addClass('success');
				photoDom.attr('src',src);
			};

			image.onerror = function(){
				dom.removeClass('success').addClass('error');
			};

			image.src = src;
		},

		/**********************************************
		 * @actionName  动画完成后的回调函数.
		 * *******************************************/
		animateComplete:function(){
			Base.prototype.animateComplete.call(this);
			this.initDom();
			this.setPhotoPos();
		}

	}));

	module.exports = c360.share.mask;
});